@@ -9,7 +9,7 @@
 %                              CCC   UUU     T                                %
 %                                                                             %
 %                                                                             %
-%                         Read DR Halo Image Format.                          %
+%                         Read DR Halo Image Format                           %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
@@ -59,9 +59,10 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
-#include "magick/quantum.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
 
 typedef struct
@@ -96,7 +97,7 @@ static void InsertRow(long depth,unsigned char *p,long y,Image *image)
   register IndexPacket *indexes;
 
 
-  index=0;
+  index=(IndexPacket) 0;
   switch (depth)
     {
     case 1:  /* Convert bitmap scanline. */
@@ -295,11 +296,10 @@ static int GetCutColors(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -333,7 +333,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -383,9 +383,14 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=Header.Width;
   image->rows=Header.Height;
   image->depth=8;
-  image->colors=1UL << (unsigned long) i;
+  image->colors=(unsigned long) (GetQuantumRange(1UL*i)+1);
 
   if (image_info->ping) goto Finish;
+  if (SetImageExtent(image,0,0) == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
 
   /* ----- Do something with palette ----- */
   if ((clone_info=CloneImageInfo(image_info)) == NULL) goto NoPalette;
@@ -409,14 +414,14 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   (void) CopyMagickString(clone_info->filename+i,".PAL",(size_t)
     (MaxTextExtent-i));
-  if((clone_info->file=MagickOpenStream(clone_info->filename,"rb"))==NULL)
+  if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
     {
       (void) CopyMagickString(clone_info->filename+i,".pal",(size_t)
         (MaxTextExtent-i));
-      if((clone_info->file=MagickOpenStream(clone_info->filename,"rb"))==NULL)
+      if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
         {
           clone_info->filename[i]='\0';
-          if((clone_info->file=MagickOpenStream(clone_info->filename,"rb"))==NULL)
+          if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
             {
               clone_info=DestroyImageInfo(clone_info);
               clone_info=NULL;
@@ -425,7 +430,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
 
-  if( (palette=AllocateImage(clone_info))==NULL ) goto NoPalette;
+  if( (palette=AcquireImage(clone_info))==NULL ) goto NoPalette;
   status=OpenBlob(clone_info,palette,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -454,7 +459,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       if(PalHeader.MaxIndex<1) goto ErasePalette;
       image->colors=PalHeader.MaxIndex+1;
-      if (AllocateImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
+      if (AcquireImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
 
       if(PalHeader.MaxRed==0) PalHeader.MaxRed=(unsigned int) QuantumRange;  /*avoid division by 0*/
       if(PalHeader.MaxGreen==0) PalHeader.MaxGreen=(unsigned int) QuantumRange;
@@ -470,20 +475,20 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (offset < 0)
                 ThrowReaderException(CorruptImageError,"ImproperImageHeader");
             }
-          image->colormap[i].red=ReadBlobLSBShort(palette);
+          image->colormap[i].red=(Quantum) ReadBlobLSBShort(palette);
           if(QuantumRange!=(Quantum) PalHeader.MaxRed)
             {
               image->colormap[i].red=RoundToQuantum(((double)
                 image->colormap[i].red*QuantumRange+(PalHeader.MaxRed>>1))/
                 PalHeader.MaxRed);
             }
-          image->colormap[i].green=ReadBlobLSBShort(palette);
+          image->colormap[i].green=(Quantum) ReadBlobLSBShort(palette);
           if(QuantumRange!=(Quantum) PalHeader.MaxGreen)
             {
               image->colormap[i].green=RoundToQuantum
                 (((double) image->colormap[i].green*QuantumRange+(PalHeader.MaxGreen>>1))/PalHeader.MaxGreen);
             }
-          image->colormap[i].blue=ReadBlobLSBShort(palette);
+          image->colormap[i].blue=(Quantum) ReadBlobLSBShort(palette);
           if(QuantumRange!=(Quantum) PalHeader.MaxBlue)
             {
               image->colormap[i].blue=RoundToQuantum
@@ -500,7 +505,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
 
       image->colors=256;
-      if (AllocateImageColormap(image,image->colors) == MagickFalse)
+      if (AcquireImageColormap(image,image->colors) == MagickFalse)
         {
         NoMemory:
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -516,7 +521,8 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
 
   /* ----- Load RLE compressed raster ----- */
-  BImgBuff=(unsigned char *) AcquireMagickMemory((size_t) ldblk);  /*Ldblk was set in the check phase*/
+  BImgBuff=(unsigned char *) AcquireQuantumMemory((size_t) ldblk,
+    sizeof(*BImgBuff));  /*Ldblk was set in the check phase*/
   if(BImgBuff==NULL) goto NoMemory;
 
   offset=SeekBlob(image,6 /*sizeof(Header)*/,SEEK_SET);
@@ -611,7 +617,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -635,20 +641,21 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterCUTImage method is:
 %
-%      RegisterCUTImage(void)
+%      unsigned long RegisterCUTImage(void)
 %
 */
-ModuleExport void RegisterCUTImage(void)
+ModuleExport unsigned long RegisterCUTImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("CUT");
-  entry->decoder=(DecoderHandler *) ReadCUTImage;
+  entry->decoder=(DecodeImageHandler *) ReadCUTImage;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("DR Halo");
   entry->module=ConstantString("CUT");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
