@@ -10,14 +10,14 @@
 %                            DDDD   IIIII  BBBB                               %
 %                                                                             %
 %                                                                             %
-%                   Read/Write Windows DIB Image Format.                      %
+%                   Read/Write Windows DIB Image Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -56,6 +56,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -127,7 +128,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image: The address of a structure of type Image.
+%    o image: the address of a structure of type Image.
 %
 %    o compression:  A value of 1 means the compressed pixels are runlength
 %      encoded for a 256-color bitmap.  A value of 2 means a 16-color bitmap.
@@ -160,9 +161,6 @@ static MagickBooleanType DecodeImage(Image *image,
   long
     y;
 
-  MagickBooleanType
-    status;
-
   register long
     i,
     x;
@@ -275,14 +273,8 @@ static MagickBooleanType DecodeImage(Image *image,
           }
         }
       }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
+      break;
   }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
@@ -312,7 +304,7 @@ static MagickBooleanType DecodeImage(Image *image,
 %
 %    o image:  The image.
 %
-%    o bytes_per_line: The number of bytes in a scanline of compressed pixels
+%    o bytes_per_line: the number of bytes in a scanline of compressed pixels
 %
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
 %      the compression process.
@@ -327,9 +319,6 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   long
     y;
 
-  MagickBooleanType
-    status;
-
   register const unsigned char
     *p;
 
@@ -371,14 +360,8 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     */
     *q++=0x00;
     *q++=0x00;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
+      break;
   }
   /*
     End of bitmap.
@@ -444,7 +427,7 @@ static MagickBooleanType IsDIB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -519,7 +502,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -527,7 +510,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   /*
-    Determine if this is a DIB file.
+    Determine if this a DIB file.
   */
   (void) ResetMagickMemory(&dib_info,0,sizeof(dib_info));
   dib_info.size=ReadBlobLSBLong(image);
@@ -592,7 +575,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Read DIB raster colormap.
       */
-      if (AllocateImageColormap(image,image->colors) == MagickFalse)
+      if (AcquireImageColormap(image,image->colors) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       length=(size_t) image->colors;
       dib_colormap=(unsigned char *) AcquireQuantumMemory(length,
@@ -693,14 +676,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(1L*image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,1L*image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -736,14 +717,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(1L*image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,1L*image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -773,14 +752,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(1L*image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,1L*image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -828,14 +805,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(1L*image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,1L*image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -863,14 +838,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(1L*image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,1L*image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -898,7 +871,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImage(image);
       image=flipped_image;
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -987,7 +960,7 @@ ModuleExport void UnregisterDIBImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -1038,7 +1011,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
   /*
     Initialize DIB raster file header.
   */
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   if (image->storage_class == DirectClass)
     {
@@ -1129,22 +1102,16 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
             }
            p++;
          }
-       if (bit != 0)
-         {
-           *q++=(unsigned char) (byte << (8-bit));
-           x++;
-         }
-       for (x=(long) (image->columns+7)/8; x < (long) bytes_per_line; x++)
-         *q++=0x00;
-       if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+         if (bit != 0)
+           {
+             *q++=(unsigned char) (byte << (8-bit));
+             x++;
+           }
+        for (x=(long) (image->columns+7)/8; x < (long) bytes_per_line; x++)
+          *q++=0x00;
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -1164,15 +1131,9 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
           *q++=(unsigned char) indexes[x];
         for ( ; x < (long) bytes_per_line; x++)
           *q++=0x00;
-        if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -1201,15 +1162,9 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
         }
         for (x=2L*image->columns; x < (long) bytes_per_line; x++)
           *q++=0x00;
-        if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -1237,21 +1192,15 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
         if (dib_info.bits_per_pixel == 24)
           for (x=3L*image->columns; x < (long) bytes_per_line; x++)
             *q++=0x00;
-        if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
   }
   if (dib_info.bits_per_pixel == 8)
-    if (image->compression != NoCompression)
+    if (image_info->compression != NoCompression)
       {
         size_t
           length;
@@ -1331,6 +1280,6 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
     }
   (void) WriteBlob(image,dib_info.image_size,pixels);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
