@@ -10,13 +10,13 @@
 %                            EEEEE  M   M  F                                  %
 %                                                                             %
 %                                                                             %
-%                  Read Windows Enahanced Metafile Format.                    %
+%                  Read Windows Enahanced Metafile Format                     %
 %                                                                             %
 %                              Software Design                                %
 %                              Bill Radcliffe                                 %
 %                                   2001                                      %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -38,7 +38,7 @@
  */
 
 #include "magick/studio.h"
-#if defined(HasWINGDI32)
+#if defined(MAGICKCORE_WINGDI32_DELEGATE)
 #  if defined(__CYGWIN__)
 #    include <windows.h>
 #  else
@@ -157,19 +157,136 @@ static MagickBooleanType IsWMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info..
+%    o image_info: the image info..
 %
 %    o exception: Return any errors or warnings in this structure.
 %
 */
 
+#if defined(MAGICKCORE_HAVE__WFOPEN)
+static size_t UTF8ToUTF16(const unsigned char *utf8,wchar_t *utf16)
+{
+  register const unsigned char
+    *p;
+
+  if (utf16 != (wchar_t *) NULL)
+    {
+      register wchar_t
+        *q;
+
+      wchar_t
+        c;
+
+      /*
+        Convert UTF-8 to UTF-16.
+      */
+      q=utf16;
+      for (p=utf8; *p != '\0'; p++)
+      {
+        if ((*p & 0x80) == 0)
+          *q=(*p);
+        else
+          if ((*p & 0xE0) == 0xC0)
+            {
+              c=(*p);
+              *q=(c & 0x1F) << 6;
+              p++;
+              if ((*p & 0xC0) != 0x80)
+                return(0);
+              *q|=(*p & 0x3F);
+            }
+          else
+            if ((*p & 0xF0) == 0xE0)
+              {
+                c=(*p);
+                *q=c << 12;
+                p++;
+                if ((*p & 0xC0) != 0x80)
+                  return(0);
+                c=(*p);
+                *q|=(c & 0x3F) << 6;
+                p++;
+                if ((*p & 0xC0) != 0x80)
+                  return(0);
+                *q|=(*p & 0x3F);
+              }
+            else
+              return(0);
+        q++;
+      }
+      *q++='\0';
+      return(q-utf16);
+    }
+  /*
+    Compute UTF-16 string length.
+  */
+  for (p=utf8; *p != '\0'; p++)
+  {
+    if ((*p & 0x80) == 0)
+      ;
+    else
+      if ((*p & 0xE0) == 0xC0)
+        {
+          p++;
+          if ((*p & 0xC0) != 0x80)
+            return(0);
+        }
+      else
+        if ((*p & 0xF0) == 0xE0)
+          {
+            p++;
+            if ((*p & 0xC0) != 0x80)
+              return(0);
+            p++;
+            if ((*p & 0xC0) != 0x80)
+              return(0);
+         }
+       else
+         return(0);
+  }
+  return(p-utf8);
+}
+
+static wchar_t *ConvertUTF8ToUTF16(const unsigned char *source)
+{
+  size_t
+    length;
+
+  wchar_t
+    *utf16;
+
+  length=UTF8ToUTF16(source,(wchar_t *) NULL);
+  if (length == 0)
+    {
+      register long
+        i;
+
+      /*
+        Not UTF-8, just copy.
+      */
+      length=strlen(source);
+      utf16=(wchar_t *) AcquireQuantumMemory(length+1,sizeof(*utf16));
+      if (utf16 == (wchar_t *) NULL)
+        return((wchar_t *) NULL);
+      for (i=0; i <= (long) length; i++)
+        utf16[i]=source[i];
+      return(utf16);
+    }
+  utf16=(wchar_t *) AcquireQuantumMemory(length+1,sizeof(*utf16));
+  if (utf16 == (wchar_t *) NULL)
+    return((wchar_t *) NULL);
+  length=UTF8ToUTF16(source,utf16);
+  return(utf16);
+}
+#endif
+
 /*
   This method reads either an enhanced metafile, a regular 16bit Windows
   metafile, or an Aldus Placeable metafile and converts it into an enhanced
   metafile.  Width and height are returned in .01mm units.
 */
-#if defined(HasWINGDI32)
-static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
+#if defined(MAGICKCORE_WINGDI32_DELEGATE)
+static HENHMETAFILE ReadEnhMetaFile(const char *path,long *width,
   long *height)
 {
 #pragma pack( push )
@@ -211,7 +328,21 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
 
   *width=512;
   *height=512;
-  hTemp=GetEnhMetaFile(szFileName);
+  hTemp=GetEnhMetaFile(path);
+#if defined(MAGICKCORE_HAVE__WFOPEN)
+  if (hTemp == (HENHMETAFILE) NULL)
+    {
+      wchar_t
+        *unicode_path;
+
+      unicode_path=ConvertUTF8ToUTF16(path);
+      if (unicode_path != (wchar_t *) NULL)
+        {
+          hTemp=GetEnhMetaFileW(unicode_path);
+          unicode_path=(wchar_t *) RelinquishMagickMemory(unicode_path);
+        }
+    }
+#endif
   if (hTemp != (HENHMETAFILE) NULL)
     {
       /*
@@ -222,7 +353,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
       *height=emfh.rclFrame.bottom-emfh.rclFrame.top;
       return(hTemp);
     }
-  hOld=GetMetaFile(szFileName);
+  hOld=GetMetaFile(path);
   if (hOld != (HMETAFILE) NULL)
     {
       /*
@@ -242,7 +373,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
         }
       if (GetMetaFileBitsEx(hOld,dwSize,pBits) == 0)
         {
-          pBits=DestroyString(pBits);
+          pBits=(BYTE *) DestroyString((char *) pBits);
           DeleteMetaFile(hOld);
           return((HENHMETAFILE) NULL);
         }
@@ -257,7 +388,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
       hTemp=SetWinMetaFileBits(dwSize,pBits,hDC,&mp);
       ReleaseDC(NULL,hDC);
       DeleteMetaFile(hOld);
-      pBits=DestroyString(pBits);
+      pBits=(BYTE *) DestroyString((char *) pBits);
       GetEnhMetaFileHeader(hTemp,sizeof(ENHMETAHEADER),&emfh);
       *width=emfh.rclFrame.right-emfh.rclFrame.left;
       *height=emfh.rclFrame.bottom-emfh.rclFrame.top;
@@ -266,8 +397,8 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
   /*
     Aldus Placeable metafile.
   */
-  hFile=CreateFile(szFileName,GENERIC_READ,0,NULL,OPEN_EXISTING,
-    FILE_ATTRIBUTE_NORMAL,NULL);
+  hFile=CreateFile(path,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,
+    NULL);
   if (hFile == INVALID_HANDLE_VALUE)
     return(NULL);
   dwSize=GetFileSize(hFile,NULL);
@@ -276,7 +407,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
   CloseHandle(hFile);
   if (((PAPMHEADER) pBits)->dwKey != 0x9ac6cdd7l)
     {
-      pBits=DestroyString(pBits);
+      pBits=(BYTE *) DestroyString((char *) pBits);
       return((HENHMETAFILE) NULL);
     }
   /*
@@ -293,7 +424,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
   hDC=GetDC(NULL);
   hTemp=SetWinMetaFileBits(dwSize,&(pBits[sizeof(APMHEADER)]),hDC,&mp);
   ReleaseDC(NULL,hDC);
-  pBits=DestroyString(pBits);
+  pBits=(BYTE *) DestroyString((char *) pBits);
   return(hTemp);
 }
 
@@ -336,7 +467,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
     *pBits,
     *ppBits;
 
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   hemf=ReadEnhMetaFile(image_info->filename,&width,&height);
   if (hemf == (HENHMETAFILE) NULL)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
@@ -506,7 +637,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   DeleteObject(hBitmap);
   return(GetFirstImageInList(image));
 }
-#endif /* HasWINGDI32 */
+#endif /* MAGICKCORE_WINGDI32_DELEGATE */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -537,7 +668,7 @@ ModuleExport unsigned long RegisterEMFImage(void)
     *entry;
 
   entry=SetMagickInfo("EMF");
-#if defined(HasWINGDI32)
+#if defined(MAGICKCORE_WINGDI32_DELEGATE)
   entry->decoder=ReadEMFImage;
 #endif
   entry->description=ConstantString(
@@ -547,7 +678,7 @@ ModuleExport unsigned long RegisterEMFImage(void)
   entry->module=ConstantString("WMF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("WMFWIN32");
-#if defined(HasWINGDI32)
+#if defined(MAGICKCORE_WINGDI32_DELEGATE)
   entry->decoder=ReadEMFImage;
 #endif
   entry->description=ConstantString("Windows WIN32 API rendered Meta File");
