@@ -10,14 +10,14 @@
 %                            P      SSSSS  22222                              %
 %                                                                             %
 %                                                                             %
-%                      Write Postscript Level II Format.                      %
+%                      Write Postscript Level II Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -56,6 +56,8 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/monitor-private.h"
 #include "magick/resource_.h"
 #include "magick/property.h"
 #include "magick/quantum-private.h"
@@ -63,7 +65,7 @@
 #include "magick/string_.h"
 #include "magick/module.h"
 #include "magick/utility.h"
-#if defined(HasTIFF)
+#if defined(MAGICKCORE_TIFF_DELEGATE)
 #define CCITTParam  "-1"
 #else
 #define CCITTParam  "0"
@@ -166,9 +168,9 @@ ModuleExport void UnregisterPS2Image(void)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
@@ -356,6 +358,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     stop;
 
   MagickBooleanType
+    progress,
     status;
 
   MagickOffsetType
@@ -417,15 +420,15 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  compression=image->compression;
+  compression=image_info->compression;
   switch (compression)
   {
-#if !defined(HasJPEG)
+#if !defined(MAGICKCORE_JPEG_DELEGATE)
     case JPEGCompression:
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JPEGLibraryIsNotAvailable","`%s'",
+        MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (JPEG)",
         image->filename);
       break;
     }
@@ -700,7 +703,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 (void) HuffmanEncodeImage(image_info,image);
                 break;
               }
-#if defined(HasTIFF)
+#if defined(MAGICKCORE_TIFF_DELEGATE)
             (void) Huffman2DEncodeImage(image_info,image);
 #endif
             break;
@@ -741,16 +744,11 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 *q++=ScaleQuantumToChar(PixelIntensityToQuantum(p));
                 p++;
               }
-              if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(SaveImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+              progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (progress == MagickFalse)
+                break;
             }
+            length=(size_t) (q-pixels);
             if (compression == LZWCompression)
               status=LZWEncodeImage(image,length,pixels);
             else
@@ -758,7 +756,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
             if (status == MagickFalse)
               {
-                CloseBlob(image);
+                (void) CloseBlob(image);
                 return(MagickFalse);
               }
             break;
@@ -781,15 +779,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                   ScaleQuantumToChar(PixelIntensityToQuantum(p)));
                 p++;
               }
-              if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(SaveImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+              progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (progress == MagickFalse)
+                break;
             }
             Ascii85Flush(image);
             break;
@@ -867,23 +859,18 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                       }
                   p++;
                 }
-                if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(SaveImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
               }
+              length=(size_t) (q-pixels);
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
               else
                 status=PackbitsEncodeImage(image,length,pixels);
               if (status == MagickFalse)
                 {
-                  CloseBlob(image);
+                  (void) CloseBlob(image);
                   return(MagickFalse);
                 }
               pixels=(unsigned char *) RelinquishMagickMemory(pixels);
@@ -930,15 +917,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                       }
                   p++;
                 }
-                if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(SaveImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
               }
               Ascii85Flush(image);
               break;
@@ -997,16 +978,11 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 indexes=GetIndexes(image);
                 for (x=0; x < (long) image->columns; x++)
                   *q++=(unsigned char) indexes[x];
-                if (image->previous == (Image *) NULL)
-                  if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                      (QuantumTick(y,image->rows) != MagickFalse))
-                    {
-                      status=image->progress_monitor(SaveImageTag,y,image->rows,
-                        image->client_data);
-                      if (status == MagickFalse)
-                        break;
-                    }
-                }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
+              }
+              length=(size_t) (q-pixels);
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
               else
@@ -1014,7 +990,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               pixels=(unsigned char *) RelinquishMagickMemory(pixels);
               if (status == MagickFalse)
                 {
-                  CloseBlob(image);
+                  (void) CloseBlob(image);
                   return(MagickFalse);
                 }
               break;
@@ -1034,15 +1010,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 indexes=GetIndexes(image);
                 for (x=0; x < (long) image->columns; x++)
                   Ascii85Encode(image,(unsigned char) indexes[x]);
-                if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(SaveImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
               }
               Ascii85Flush(image);
               break;
@@ -1067,14 +1037,10 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 1)
@@ -1089,6 +1055,6 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
