@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -78,19 +79,28 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+static inline unsigned long MagickMin(const unsigned long x,
+  const unsigned long y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define GetBit(alpha,i) (((unsigned long) (alpha) >> (unsigned long) \
-  (i)) & 0x01)
-#define SetBit(alpha,i,set) (alpha)=(Quantum) ((set) != 0 ? (unsigned long) \
-  (alpha) | (1UL << (unsigned long) (i)) : (unsigned long) (alpha) & \
-  ~(1UL << (unsigned long) (i)))
+#define GetBit(alpha,i) MagickMin((((unsigned long) (alpha) >> (unsigned long) \
+  (i)) & 0x01),16)
+#define SetBit(alpha,i,set) (alpha)=(IndexPacket) ((set) != 0 ? \
+  (unsigned long) (alpha) | (1UL << (unsigned long) (i)) : (unsigned long) \
+  (alpha) & ~(1UL << (unsigned long) (i)))
 
   Image
     *image,
@@ -131,7 +141,7 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   read_info=CloneImageInfo(image_info);
@@ -141,12 +151,12 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   read_info=DestroyImageInfo(read_info);
   if (watermark == (Image *) NULL)
     return((Image *) NULL);
-  watermark->depth=QuantumDepth;
-  if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
+  watermark->depth=MAGICKCORE_QUANTUM_DEPTH;
+  if (AcquireImageColormap(image,MaxColormapSize) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   /*
@@ -155,12 +165,13 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   c=0;
   i=0;
   j=0;
-  k=image->offset;
-  for (i=QuantumDepth-1; (i >= 0) && (j < QuantumDepth); i--)
+  i=MAGICKCORE_QUANTUM_DEPTH-1;
+  for (k=image->offset; (i >= 0) && (j < MAGICKCORE_QUANTUM_DEPTH); i--)
   {
-    for (y=0; (y < (long) image->rows) && (j < QuantumDepth); y++)
+    for (y=0; (y < (long) image->rows) && (j < MAGICKCORE_QUANTUM_DEPTH); y++)
     {
-      for (x=0; (x < (long) image->columns) && (j < QuantumDepth); x++)
+      x=0;
+      for (; (x < (long) image->columns) && (j < MAGICKCORE_QUANTUM_DEPTH); x++)
       {
         pixel=AcquireOnePixel(watermark,k % (long) watermark->columns,
           k/(long) watermark->columns,exception);
@@ -198,14 +209,9 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
           j++;
       }
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(i,QuantumDepth) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImagesTag,i,QuantumDepth,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,LoadImagesTag,i,MAGICKCORE_QUANTUM_DEPTH);
+    if (status == MagickFalse)
+      break;
   }
   watermark=DestroyImage(watermark);
   (void) SyncImage(image);
@@ -242,6 +248,7 @@ ModuleExport unsigned long RegisterSTEGANOImage(void)
 
   entry=SetMagickInfo("STEGANO");
   entry->decoder=(DecodeImageHandler *) ReadSTEGANOImage;
+  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Steganographic image");
   entry->module=ConstantString("STEGANO");
   (void) RegisterMagickInfo(entry);
