@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -103,13 +103,11 @@ typedef struct _CacheMethods
     sync_pixel_handler;
 } CacheMethods;
 
-typedef struct _NexusInfo NexusInfo;
+typedef struct _NexusInfo
+   NexusInfo;
 
 typedef struct _CacheInfo
 {
-  unsigned long
-    id;
-
   NexusInfo
     *nexus_info;
 
@@ -122,7 +120,7 @@ typedef struct _CacheInfo
   ColorspaceType
     colorspace;
 
-  CacheType
+  volatile CacheType
     type;
 
   MagickBooleanType
@@ -166,14 +164,22 @@ typedef struct _CacheInfo
   MagickBooleanType
     debug;
 
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  pthread_t
+#elif defined(__WINDOWS__)
+  DWORD
+#else
   unsigned long
-    thread;
+#endif
+    id;
 
-  long
+  volatile long
     reference_count;
 
   SemaphoreInfo
-    *semaphore;
+    *semaphore,
+    *nexus_semaphore,
+    *disk_semaphore;
 
   unsigned long
     signature;
