@@ -3,21 +3,21 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%        CCCC   OOO  M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE          %
-%       C      O   O MM MM  P   P  O   O  SS       I      T    E              %
-%       C      O   O M M M  PPPP   O   O   SSS     I      T    EEE            %
-%       C      O   O M   M  P      O   O     SS    I      T    E              %
-%        CCCC   OOO  M   M  P       OOO   SSSSS  IIIII    T    EEEEE          %
+%        CCCC   OOO   M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE         %
+%       C      O   O  MM MM  P   P  O   O  SS       I      T    E             %
+%       C      O   O  M M M  PPPP   O   O   SSS     I      T    EEE           %
+%       C      O   O  M   M  P      O   O     SS    I      T    E             %
+%        CCCC   OOO   M   M  P       OOO   SSSSS  IIIII    T    EEEEE         %
 %                                                                             %
 %                                                                             %
-%                    ImageMagick Image Composite Methods                      %
+%                     MagickCore Image Composite Methods                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -59,6 +59,8 @@
 #include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/log.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/memory_.h"
 #include "magick/option.h"
 #include "magick/pixel-private.h"
@@ -95,19 +97,19 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o channel: The channel.
+%    o channel: the channel.
 %
 %    o compose: This operator affects how the composite is applied to
 %      the image.  The operators and how they are utilized are listed here
 %      http://www.w3.org/TR/SVG12/#compositing.
 %
-%    o composite_image: The composite image.
+%    o composite_image: the composite image.
 %
-%    o x_offset: The column offset of the composited image.
+%    o x_offset: the column offset of the composited image.
 %
-%    o y_offset: The row offset of the composited image.
+%    o y_offset: the row offset of the composited image.
 %
 */
 
@@ -550,7 +552,7 @@ static inline void CompositeLighten(const MagickPixelPacket *p,
 
 static inline MagickRealType LinearLight(const MagickRealType p,
   const MagickRealType alpha,const MagickRealType q,const MagickRealType beta)
-{ 
+{
   return((1.0-QuantumScale*beta)*q+2.0*(1.0-QuantumScale*alpha)*p-1.0);
 }
 
@@ -569,7 +571,7 @@ static inline void CompositeLinearLight(const MagickPixelPacket *p,
   composite->blue=gamma*LinearLight(p->blue,alpha,q->blue,beta);
   if (q->colorspace == CMYKColorspace)
     composite->index=gamma*LinearLight(p->index,alpha,q->index,beta);
-}   
+}
 
 static inline MagickRealType Minus(const MagickRealType p,
   const MagickRealType alpha,const MagickRealType q,const MagickRealType beta)
@@ -989,17 +991,13 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   const ChannelType magick_unused(channel),const CompositeOperator compose,
   const Image *composite_image,const long x_offset,const long y_offset)
 {
+#define CompositeImageTag  "Composite/Image"
+
   const char
     *value;
 
-  const PixelPacket
-    *pixels;
-
   double
-    brightness,
-    hue,
-    sans,
-    saturation;
+    sans;
 
   GeometryInfo
     geometry_info;
@@ -1007,20 +1005,16 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   Image
     *displace_image;
 
-  IndexPacket
-    *composite_indexes,
-    *indexes;
-
   long
+    progress,
     y;
 
   MagickBooleanType
-    modify_outside_overlay;
+    modify_outside_overlay,
+    status;
 
   MagickPixelPacket
-    composite,
-    destination,
-    source;
+    zero;
 
   MagickRealType
     amount,
@@ -1035,18 +1029,9 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   MagickStatusType
     flags;
 
-  register const PixelPacket
-    *p;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
   ViewInfo
-    *composite_view,
-    *image_view;
+    **composite_view,
+    **image_view;
 
   /*
     Prepare composite image.
@@ -1061,6 +1046,7 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
+  GetMagickPixelPacket(image,&zero);
   displace_image=(Image *) NULL;
   amount=0.5;
   destination_dissolve=1.0;
@@ -1094,7 +1080,7 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         channel to exist, to add transparency.
       */
       if (image->matte == MagickFalse)
-        (void) SetImageOpacity(image,OpaqueOpacity);
+        (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       modify_outside_overlay=MagickTrue;
       break;
     }
@@ -1126,12 +1112,12 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
       /*
         Allocate the displace image.
       */
-      displace_image=CloneImage(composite_image,0,0,MagickTrue,
-        &image->exception);
+      displace_image=CloneImage(composite_image,composite_image->columns,
+        composite_image->rows,MagickTrue,&image->exception);
       if (displace_image == (Image *) NULL)
         return(MagickFalse);
-      horizontal_scale=20.0;
-      vertical_scale=20.0;
+      horizontal_scale=(MagickRealType) composite_image->columns;
+      vertical_scale=(MagickRealType) composite_image->rows;
       if (composite_image->geometry != (char *) NULL)
         {
           /*
@@ -1147,19 +1133,28 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
       /*
         Shift image pixels as defined by a displacement map.
       */
-      GetMagickPixelPacket(displace_image,&pixel);
+      pixel=zero;
       resample_filter=AcquireResampleFilter(image,&image->exception);
-      image_view=OpenCacheView(image);
-      displace_view=OpenCacheView(displace_image);
-      composite_view=OpenCacheView(composite_image);
+      image_view=AcquireCacheView(image);
+      displace_view=AcquireCacheView(displace_image);
+      composite_view=AcquireCacheView(composite_image);
       for (y=0; y < (long) composite_image->rows; y++)
       {
+        register const PixelPacket
+          *p;
+
+        register long
+          x;
+
+        register PixelPacket
+          *q;
+
         if (((y+y_offset) < 0) || ((y+y_offset) >= (long) image->rows))
           continue;
         p=AcquireCacheViewPixels(composite_view,0,y,composite_image->columns,1,
           &image->exception);
         q=GetCacheViewPixels(image_view,0,y+y_offset,image->columns,1);
-        r=GetCacheViewPixels(displace_view,0,y,displace_image->columns,1);
+        r=SetCacheViewPixels(displace_view,0,y,displace_image->columns,1);
         if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL) ||
             (r == (PixelPacket *) NULL))
           break;
@@ -1173,31 +1168,27 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
               q++;
               continue;
             }
-          x_displace=(horizontal_scale*(PixelIntensityToQuantum(p)-
+          x_displace=(horizontal_scale*(p->red -
+            (((MagickRealType) QuantumRange+1.0)/2.0)))/
+            (((MagickRealType) QuantumRange+1.0)/2.0);
+          y_displace=(vertical_scale*(p->green -
             (((MagickRealType) QuantumRange+1.0)/2.0)))/
             (((MagickRealType) QuantumRange+1.0)/2.0);
-          if (composite_image->matte != MagickFalse)
-            y_displace=(-vertical_scale*(p->opacity-(((MagickRealType)
-              QuantumRange+1.0)/2.0)))/(((MagickRealType) QuantumRange+1.0)/
-              2.0);
-          else
-            y_displace=(vertical_scale*(PixelIntensityToQuantum(p)-
-              (((MagickRealType) QuantumRange+1.0)/2.0)))/
-              (((MagickRealType) QuantumRange+1.0)/2.0);
           pixel=ResamplePixelColor(resample_filter,(double)
             (x_offset+x+x_displace),(double) (y_offset+y+y_displace));
+          /* Add transparency handling here */
           SetPixelPacket(displace_image,&pixel,r,displace_indexes+x);
           p++;
           q++;
           r++;
         }
-        if (SyncCacheView(displace_view) == MagickFalse)
+        if (SyncCacheViewPixels(displace_view) == MagickFalse)
           break;
       }
       resample_filter=DestroyResampleFilter(resample_filter);
-      composite_view=CloseCacheView(composite_view);
-      displace_view=CloseCacheView(displace_view);
-      image_view=CloseCacheView(image_view);
+      composite_view=DestroyCacheView(composite_view);
+      displace_view=DestroyCacheView(displace_view);
+      image_view=DestroyCacheView(image_view);
       composite_image=displace_image;
       break;
     }
@@ -1285,22 +1276,54 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   /*
     Composite image.
   */
-  hue=0.0;
-  saturation=0.0;
-  brightness=0.0;
+  status=MagickTrue;
+  progress=0;
   midpoint=((MagickRealType) QuantumRange+1.0)/2;
-  GetMagickPixelPacket(composite_image,&source);
-  GetMagickPixelPacket(image,&destination);
-  image_view=OpenCacheView(image);
-  composite_view=OpenCacheView(composite_image);
+  image_view=AcquireCacheViewThreadSet(image);
+  composite_view=AcquireCacheViewThreadSet(composite_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
+    const PixelPacket
+      *pixels;
+
+    double
+      brightness,
+      hue,
+      saturation;
+
+    MagickPixelPacket
+      composite,
+      destination,
+      source;
+
+    register const IndexPacket
+      *composite_indexes;
+
+    register const PixelPacket
+      *p;
+
+    register IndexPacket
+      *indexes;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
     if (modify_outside_overlay == MagickFalse)
       {
         if (y < y_offset)
           continue;
         if ((y-y_offset) >= (long) composite_image->rows)
-          break;
+          continue;
       }
     /*
       If pixels is NULL, y is outside overlay region.
@@ -1309,19 +1332,30 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     p=(PixelPacket *) NULL;
     if ((y >= y_offset) && ((y-y_offset) < (long) composite_image->rows))
       {
-        p=AcquireCacheViewPixels(composite_view,0,y-y_offset,composite_image->columns,
-          1,&image->exception);
+        p=AcquireCacheViewPixels(composite_view[id],0,y-y_offset,
+          composite_image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
-          break;
+          {
+            status=MagickFalse;
+            continue;
+          }
         pixels=p;
         if (x_offset < 0)
           p-=x_offset;
       }
-    q=GetCacheViewPixels(image_view,0,y,image->columns,1);
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetCacheViewIndexes(image_view);
-    composite_indexes=GetCacheViewIndexes(composite_view);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewIndexes(image_view[id]);
+    composite_indexes=AcquireCacheViewIndexes(composite_view[id]);
+    GetMagickPixelPacket(composite_image,&source);
+    GetMagickPixelPacket(image,&destination);
+    hue=0.0;
+    saturation=0.0;
+    brightness=0.0;
     for (x=0; x < (long) image->columns; x++)
     {
       if (modify_outside_overlay == MagickFalse)
@@ -1344,7 +1378,7 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
           destination.red=(MagickRealType) QuantumRange-destination.red;
           destination.green=(MagickRealType) QuantumRange-destination.green;
           destination.blue=(MagickRealType) QuantumRange-destination.blue;
-          destination.index=(MagickRealType) QuantumRange-indexes[x];
+          destination.index=(MagickRealType) (QuantumRange-indexes[x]);
         }
       /*
         Handle destination modifications outside overlaid region.
@@ -1381,7 +1415,6 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
             }
             default:
               break;
-
           }
           if (image->colorspace == CMYKColorspace)
             {
@@ -1761,7 +1794,7 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         case CopyBlackCompositeOp:
         {
           if (source.colorspace != CMYKColorspace)
-            ConvertRGBToCMYK(&source); 
+            ConvertRGBToCMYK(&source);
           composite.index=source.index;
           break;
         }
@@ -1778,8 +1811,7 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
       q->red=RoundToQuantum(composite.red);
       q->green=RoundToQuantum(composite.green);
       q->blue=RoundToQuantum(composite.blue);
-      if (image->matte != MagickFalse)
-        q->opacity=RoundToQuantum(composite.opacity);
+      q->opacity=RoundToQuantum(composite.opacity);
       if (image->colorspace == CMYKColorspace)
         indexes[x]=RoundToQuantum(composite.index);
       p++;
@@ -1787,12 +1819,25 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         p=pixels;
       q++;
     }
-    if (SyncCacheView(image_view) == MagickFalse)
-      break;
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,CompositeImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
   }
-  composite_view=CloseCacheView(composite_view);
-  image_view=CloseCacheView(image_view);
+  composite_view=DestroyCacheViewThreadSet(composite_view);
+  image_view=DestroyCacheViewThreadSet(image_view);
   if (displace_image != (Image * ) NULL)
     displace_image=DestroyImage(displace_image);
-  return(MagickTrue);
+  return(status);
 }
