@@ -9,14 +9,14 @@
 %           DDDD   EEEEE  LLLLL  EEEEE   GGG   A   A    T    EEEEE            %
 %                                                                             %
 %                                                                             %
-%                   Methods to Read/Write/Invoke Delegates                    %
+%             MagickCore Methods to Read/Write/Invoke Delegates               %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -77,8 +77,9 @@ static const char
     "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
     "<delegatemap>"
     "  <delegate decode=\"autotrace\" stealth=\"True\" command=\"&quot;autotrace&quot; -output-format svg -output-file &quot;%o&quot; &quot;%i&quot;\"/>"
-    "  <delegate decode=\"browse\" stealth=\"True\" spawn=\"True\" command=\"&quot;htmlview&quot; http://www.imagemagick.org/\"/>"
-    "  <delegate decode=\"cgm\" thread-support=\"False\" command=\"&quot;ralcgm&quot; -d ps -oC &lt; &quot;%i&quot; &gt; &quot;%o&quot; &gt;&gt;/dev/null\"/>"
+    "  <delegate decode=\"avi:decode\" stealth=\"True\" command=\"&quot;mplayer&quot; &quot;%i&quot; -really-quiet -ao null -vo png:z=3\"/>"
+    "  <delegate decode=\"browse\" stealth=\"True\" spawn=\"True\" command=\"&quot;xdg-open&quot; http://www.imagemagick.org/\"/>"
+    "  <delegate decode=\"cgm\" thread-support=\"False\" command=\"&quot;ralcgm&quot; -d ps -oC &lt; &quot;%i&quot; &gt; &quot;%o&quot; 2&gt; &quot;%u&quot;\"/>"
     "  <delegate decode=\"dng:decode\" command=\"&quot;/usr/bin/ufraw-batch&quot; --silent --wb=camera --black-point=auto --exposure=auto --create-id=also --out-type=ppm16 &quot;--output=%u.pnm&quot; &quot;%i&quot;\"/>"
     "  <delegate decode=\"edit\" stealth=\"True\" command=\"&quot;xterm&quot; -title &quot;Edit Image Comment&quot; -e vi &quot;%o&quot;\"/>"
     "  <delegate decode=\"eps\" encode=\"pdf\" mode=\"bi\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pdfwrite&quot; &quot;-sOutputFile=%o&quot; &quot;-f%i&quot;\"/>"
@@ -93,11 +94,11 @@ static const char
     "  <delegate decode=\"https\" command=\"&quot;wget&quot; -q -O &quot;%o&quot; &quot;https:%M&quot;\"/>"
     "  <delegate decode=\"ilbm\" command=\"&quot;ilbmtoppm&quot; &quot;%i&quot; &gt; &quot;%o&quot;\"/>"
     "  <delegate decode=\"man\" command=\"&quot;groff&quot; -man -Tps &quot;%i&quot; &gt; &quot;%o&quot;\"/>"
-    "  <delegate decode=\"mpeg:decode\" stealth=\"True\" command=\"&quot;mpeg2decode&quot; -q -b &quot;%i&quot; -f -r -o3 &quot;%u%%d&quot;\"/>"
-    "  <delegate encode=\"mpeg:encode\" stealth=\"True\" command=\"&quot;mpeg2encode&quot; &quot;%i&quot; &quot;%o&quot;\"/>"
-    "  <delegate decode=\"pcl:color\" stealth=\"True\" command=\"&quot;pcl6&quot; -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=ppmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
-    "  <delegate decode=\"pcl:cmyk\" stealth=\"True\" command=\"&quot;pcl6&quot; -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=bmpsep8&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
-    "  <delegate decode=\"pcl:mono\" stealth=\"True\" command=\"&quot;pcl6&quot; -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pbmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
+    "  <delegate decode=\"mpeg:decode\" stealth=\"True\" command=\"&quot;ffmpeg&quot; --i &quot;%i&quot; -vcodec pam -an -f rawvideo -y &quot;%u0.pam&quot; 2;&gt; &quot;%Z&quot;\"/>"
+    "  <delegate decode=\"null\" encode=\"mpeg:encode\" stealth=\"True\" command=\"&quot;ffmpeg&quot; &quot;%M%%d.jpg&quot; &quot;%u.%m&quot; 2;&gt; &quot;%Z&quot;\"/>"
+    "  <delegate decode=\"pcl:color\" stealth=\"True\" command=\"&quot;pcl6&quot; -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=ppmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
+    "  <delegate decode=\"pcl:cmyk\" stealth=\"True\" command=\"&quot;pcl6&quot; -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=bmpsep8&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
+    "  <delegate decode=\"pcl:mono\" stealth=\"True\" command=\"&quot;pcl6&quot; -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pbmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
     "  <delegate decode=\"pdf\" encode=\"eps\" mode=\"bi\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=epswrite&quot; &quot;-sOutputFile=%o&quot; &quot;-f%i&quot;\"/>"
     "  <delegate decode=\"pdf\" encode=\"ps\" mode=\"bi\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pswrite&quot; &quot;-sOutputFile=%o&quot; &quot;-f%i&quot;\"/>"
     "  <delegate decode=\"pic\" command=\"&quot;ra_pfm&quot; &quot;%i&quot; &quot;%o&quot;\"/>"
@@ -107,11 +108,11 @@ static const char
     "  <delegate decode=\"ps\" encode=\"eps\" mode=\"bi\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=epswrite&quot; &quot;-sOutputFile=%o&quot; &quot;-f%i&quot;\"/>"
     "  <delegate decode=\"ps\" encode=\"pdf\" mode=\"bi\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pdfwrite&quot; &quot;-sOutputFile=%o&quot; &quot;-f%i&quot;\"/>"
     "  <delegate decode=\"ps\" encode=\"print\" mode=\"encode\" command=\"lpr &quot;%i&quot;\"/>"
-    "  <delegate decode=\"ps:alpha\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pngalpha&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
-    "  <delegate decode=\"ps:bbox\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=bbox&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
-    "  <delegate decode=\"ps:cmyk\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pam&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
-    "  <delegate decode=\"ps:color\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pnmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
-    "  <delegate decode=\"ps:mono\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pnmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-g%s&quot; &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
+    "  <delegate decode=\"ps:alpha\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pngalpha&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
+    "  <delegate decode=\"ps:bbox\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=bbox&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
+    "  <delegate decode=\"ps:cmyk\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pam&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
+    "  <delegate decode=\"ps:color\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pnmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
+    "  <delegate decode=\"ps:mono\" stealth=\"True\" command=\"&quot;gs&quot; -q -dQUIET -dSAFER -dPARANOIDSAFE -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=0 &quot;-sDEVICE=pnmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;-f%s&quot; &quot;-f%s&quot;\"/>"
     "  <delegate decode=\"rad\" command=\"&quot;ra_pfm&quot; &quot;%i&quot; &quot;%o&quot;\"/>"
     "  <delegate decode=\"rgba\" encode=\"rle\" mode=\"encode\" command=\"&quot;rawtorle&quot; -o &quot;%o&quot; -v &quot;%i&quot;\"/>"
     "  <delegate decode=\"scan\" command=\"&quot;scanimage&quot; -d &quot;%i&quot; &gt; &quot;%o&quot;\"/>"
@@ -187,7 +188,7 @@ MagickExport void DestroyDelegateList(void)
     delegate_list=DestroyLinkedList(delegate_list,DestroyDelegate);
   instantiate_delegate=MagickFalse;
   RelinquishSemaphoreInfo(delegate_semaphore);
-  delegate_semaphore=DestroySemaphoreInfo(delegate_semaphore);
+  DestroySemaphoreInfo(&delegate_semaphore);
 }
 
 /*
@@ -214,9 +215,9 @@ MagickExport void DestroyDelegateList(void)
 %    o command: Method GetDelegateCommand returns the command associated
 %      with specified delegate tag.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o decode: Specifies the decode delegate we are searching for as a
 %      character string.
@@ -266,7 +267,7 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
     (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
       "MemoryAllocationFailed","`%s'",commands[0]);
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (i=0; commands[i] != (char *) NULL; i++)
     commands[i]=DestroyString(commands[i]);
@@ -382,8 +383,11 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
         break;
     p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   }
+  if (p != (const DelegateInfo *) NULL)
+    (void) InsertValueInLinkedList(delegate_list,0,
+      RemoveElementByValueFromLinkedList(delegate_list,p));
   RelinquishSemaphoreInfo(delegate_semaphore);
-  return( p);
+  return(p);
 }
 
 /*
@@ -710,9 +714,9 @@ static MagickBooleanType InitializeDelegateList(ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The imageInfo.
+%    o image_info: the imageInfo.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -725,13 +729,19 @@ static inline size_t MagickMin(const size_t x,const size_t y)
   return(y);
 }
 
-static void CopyDelegateFile(const char *source,const char *destination)
+static MagickBooleanType CopyDelegateFile(const char *source,
+  const char *destination)
 {
   int
     destination_file,
-    source_file,
+    source_file;
+
+  MagickBooleanType
     status;
 
+  register size_t
+    i;
+
   size_t
     length,
     quantum;
@@ -740,7 +750,7 @@ static void CopyDelegateFile(const char *source,const char *destination)
     count;
 
   struct stat
-    file_info;
+    attributes;
 
   unsigned char
     *buffer;
@@ -750,32 +760,33 @@ static void CopyDelegateFile(const char *source,const char *destination)
   */
   assert(source != (const char *) NULL);
   assert(destination != (char *) NULL);
-  status=stat(destination,&file_info);
-  if ((status == 0) && (file_info.st_size != 0))
-    return;
+  status=GetPathAttributes(destination,&attributes);
+  if ((status != MagickFalse) && (attributes.st_size != 0))
+    return(MagickTrue);
   /*
     Copy source file to destination.
   */
   destination_file=open(destination,O_WRONLY | O_BINARY | O_CREAT,S_MODE);
   if (destination_file == -1)
-    return;
+    return(MagickFalse);
   source_file=open(source,O_RDONLY | O_BINARY);
   if (source_file == -1)
     {
       (void) close(destination_file);
-      return;
+      return(MagickFalse);
     }
-  quantum=(size_t) MagickMaxBufferSize;
-  if ((fstat(source_file,&file_info) == 0) && (file_info.st_size != 0))
-    quantum=MagickMin((size_t) file_info.st_size,MagickMaxBufferSize);
+  quantum=(size_t) MagickMaxBufferExtent;
+  if ((fstat(source_file,&attributes) == 0) && (attributes.st_size != 0))
+    quantum=MagickMin((size_t) attributes.st_size,MagickMaxBufferExtent);
   buffer=(unsigned char *) AcquireQuantumMemory(quantum,sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     {
       (void) close(source_file);
       (void) close(destination_file);
-      return;
+      return(MagickFalse);
     }
-  for (length=0; ; )
+  length=0;
+  for (i=0; ; i+=count)
   {
     count=(ssize_t) read(source_file,buffer,quantum);
     if (count <= 0)
@@ -788,6 +799,7 @@ static void CopyDelegateFile(const char *source,const char *destination)
   (void) close(destination_file);
   (void) close(source_file);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  return(i != 0 ? MagickTrue : MagickFalse);
 }
 
 MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
@@ -849,9 +861,9 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
     }
   if ((delegate_info->mode != 0) &&
       (((decode != (const char *) NULL) &&
-       (delegate_info->encode != (char *) NULL)) ||
-      ((encode != (const char *) NULL) &&
-       (delegate_info->decode != (char *) NULL))))
+        (delegate_info->encode != (char *) NULL)) ||
+       ((encode != (const char *) NULL) &&
+        (delegate_info->decode != (char *) NULL))))
     {
       char
         *magick;
@@ -894,7 +906,8 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       clone_info=CloneImageInfo(image_info);
       (void) CopyMagickString((char *) clone_info->magick,magick,
         MaxTextExtent);
-      (void) CopyMagickString(image->magick,magick,MaxTextExtent);
+      if (LocaleCompare(magick,"NULL") != 0)
+        (void) CopyMagickString(image->magick,magick,MaxTextExtent);
       magick=DestroyString(magick);
       (void) FormatMagickString(clone_info->filename,MaxTextExtent,"%s:",
         delegate_info->decode);
@@ -959,43 +972,56 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
           "UnableToCreateTemporaryFile",image_info->zero);
         break;
       }
-    status=AcquireUniqueSymbolicLink(input_filename,image->filename);
-    if (status == MagickFalse)
+    if (LocaleCompare(decode,"SCAN") != 0)
       {
-        ThrowFileException(exception,FileOpenError,
-          "UnableToCreateTemporaryFile",input_filename);
-        break;
+        status=AcquireUniqueSymbolicLink(input_filename,image->filename);
+        if (status == MagickFalse)
+          {
+            ThrowFileException(exception,FileOpenError,
+              "UnableToCreateTemporaryFile",input_filename);
+            break;
+          }
       }
+    status=MagickFalse;
     command=InterpretImageProperties(image_info,image,commands[i]);
-    if (command == (char *) NULL)
-      break;
-    /*
-      Execute delegate.
-    */
-    if (delegate_info->spawn != MagickFalse)
-      (void) ConcatenateString(&command," &");
-    status=SystemCommand(image_info->verbose,command) != 0 ?  MagickTrue :
-      MagickFalse;
-    if (delegate_info->spawn != MagickFalse)
-      (void) sleep(2);
-    command=DestroyString(command);
-    (void) RelinquishUniqueFileResource(image->filename);
+    if (command != (char *) NULL)
+      {
+        /*
+          Execute delegate.
+        */
+        if (delegate_info->spawn != MagickFalse)
+          (void) ConcatenateString(&command," &");
+        status=SystemCommand(image_info->verbose,command) != 0 ?  MagickTrue :
+          MagickFalse;
+        if (delegate_info->spawn != MagickFalse)
+          (void) sleep(2);
+        command=DestroyString(command);
+      }
+    if (LocaleCompare(decode,"SCAN") != 0)
+      {
+        if (CopyDelegateFile(image->filename,input_filename) == MagickFalse)
+          (void) RelinquishUniqueFileResource(input_filename);
+      }
+    if (CopyDelegateFile(image_info->filename,output_filename) == MagickFalse)
+      (void) RelinquishUniqueFileResource(output_filename);
+    if (image_info->temporary != MagickFalse)
+      (void) RelinquishUniqueFileResource(image_info->filename);
     (void) RelinquishUniqueFileResource(image_info->unique);
     (void) RelinquishUniqueFileResource(image_info->zero);
-    CopyDelegateFile(image_info->filename,output_filename);
     (void) RelinquishUniqueFileResource(image_info->filename);
+    (void) RelinquishUniqueFileResource(image->filename);
     if (status != MagickFalse)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
           "DelegateFailed","`%s'",commands[i]);
-        return(MagickFalse);
+        break;
       }
     commands[i]=DestroyString(commands[i]);
   }
   (void) CopyMagickString(image_info->filename,output_filename,MaxTextExtent);
   (void) CopyMagickString(image->filename,input_filename,MaxTextExtent);
   /*
-    Free resources.
+    Relinquish resources.
   */
   for ( ; commands[i] != (char *) NULL; i++)
     commands[i]=DestroyString(commands[i]);
@@ -1066,7 +1092,7 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
       {
         if (delegate_info[i]->path != (char *) NULL)
           (void) fprintf(file,"\nPath: %s\n\n",delegate_info[i]->path);
-        (void) fprintf(file,"Delegate             Command\n");
+        (void) fprintf(file,"Delegate                Command\n");
         (void) fprintf(file,"-------------------------------------------------"
           "------------------------------\n");
       }
@@ -1079,7 +1105,7 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
     commands=StringToList(delegate_info[i]->commands);
     if (commands == (char **) NULL)
       continue;
-    (void) fprintf(file,"%8s%c=%c%s  ",delegate_info[i]->decode ?
+    (void) fprintf(file,"%11s%c=%c%s  ",delegate_info[i]->decode ?
       delegate_info[i]->decode : "",delegate_info[i]->mode <= 0 ? '<' : ' ',
       delegate_info[i]->mode >= 0 ? '>' : ' ',delegate);
     StripString(commands[0]);
@@ -1132,25 +1158,24 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
 static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
+  char
+    keyword[MaxTextExtent],
+    *token;
+
   const char
-    *attribute;
+    *q;
 
   DelegateInfo
-    *delegate_info = (DelegateInfo *) NULL;
+    *delegate_info;
 
   MagickBooleanType
     status;
 
-  XMLTreeInfo
-    *delegate,
-    *delegate_map,
-    *include;
-
   /*
     Load the delegate map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading delegate map \"%s\" ...",filename);
+    "Loading delegate configuration file \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (delegate_list == (LinkedListInfo *) NULL)
@@ -1163,116 +1188,203 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
-  delegate_map=NewXMLTree(xml,exception);
-  if (delegate_map == (XMLTreeInfo *) NULL)
-    return(MagickFalse);
   status=MagickTrue;
-  include=GetXMLTreeChild(delegate_map,"include");
-  while (include != (XMLTreeInfo *) NULL)
+  delegate_info=(DelegateInfo *) NULL;
+  token=AcquireString(xml);
+  for (q=(char *) xml; *q != '\0'; )
   {
     /*
-      Process include element.
+      Interpret XML.
     */
-    attribute=GetXMLTreeAttribute(include,"file");
-    if (attribute != (const char *) NULL)
+    GetMagickToken(q,&q,token);
+    if (*token == '\0')
+      break;
+    (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
       {
-        if (depth > 200)
-          (void) ThrowMagickException(exception,GetMagickModule(),
-            DelegateError,"IncludeElementNestedTooDeeply","`%s'",filename);
-        else
-          {
-            char
-              path[MaxTextExtent],
-              *xml;
-
-            GetPathComponent(filename,HeadPath,path);
-            if (*path != '\0')
-              (void) ConcatenateMagickString(path,DirectorySeparator,
-                MaxTextExtent);
-            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
-            xml=FileToString(path,~0,exception);
-            if (xml != (char *) NULL)
-              {
-                status=LoadDelegateList(xml,path,depth+1,exception);
-                xml=DestroyString(xml);
-              }
-          }
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
       }
-    include=GetNextXMLTreeTag(include);
-  }
-  delegate=GetXMLTreeChild(delegate_map,"delegate");
-  while (delegate != (XMLTreeInfo *) NULL)
-  {
-    const char
-      *attribute;
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
+      {
+        /*
+          Comment element.
+        */
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
+    if (LocaleCompare(keyword,"<include") == 0)
+      {
+        /*
+          Include element.
+        */
+        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
+        {
+          (void) CopyMagickString(keyword,token,MaxTextExtent);
+          GetMagickToken(q,&q,token);
+          if (*token != '=')
+            continue;
+          GetMagickToken(q,&q,token);
+          if (LocaleCompare(keyword,"file") == 0)
+            {
+              if (depth > 200)
+                (void) ThrowMagickException(exception,GetMagickModule(),
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
+              else
+                {
+                  char
+                    path[MaxTextExtent],
+                    *xml;
 
-    /*
-      Process delegate element.
-    */
-    delegate_info=(DelegateInfo *) AcquireMagickMemory(sizeof(*delegate_info));
+                  GetPathComponent(filename,HeadPath,path);
+                  if (*path != '\0')
+                    (void) ConcatenateMagickString(path,DirectorySeparator,
+                      MaxTextExtent);
+                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  xml=FileToString(path,~0,exception);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadDelegateList(xml,path,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
+                }
+            }
+        }
+        continue;
+      }
+    if (LocaleCompare(keyword,"<delegate") == 0)
+      {
+        /*
+          Delegate element.
+        */
+        delegate_info=(DelegateInfo *) AcquireMagickMemory(
+          sizeof(*delegate_info));
+        if (delegate_info == (DelegateInfo *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+        (void) ResetMagickMemory(delegate_info,0,sizeof(*delegate_info));
+        delegate_info->path=ConstantString(filename);
+        delegate_info->signature=MagickSignature;
+        continue;
+      }
     if (delegate_info == (DelegateInfo *) NULL)
-      ThrowFatalException(ResourceLimitFatalError,
-        "MemoryAllocationFailed");
-    (void) ResetMagickMemory(delegate_info,0,sizeof(*delegate_info));
-    delegate_info->path=ConstantString(filename);
-    delegate_info->signature=MagickSignature;
-    attribute=GetXMLTreeAttribute(delegate,"command");
-    if (attribute != (const char *) NULL)
+      continue;
+    if (LocaleCompare(keyword,"/>") == 0)
       {
-        char
-          *commands;
-
-        commands=ConstantString(attribute);
-#if defined(__WINDOWS__)
-        if (strchr(commands,'@') != (char *) NULL)
+        status=AppendValueToLinkedList(delegate_list,delegate_info);
+        if (status == MagickFalse)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            delegate_info->commands);
+        delegate_info=(DelegateInfo *) NULL;
+      }
+    GetMagickToken(q,(const char **) NULL,token);
+    if (*token != '=')
+      continue;
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    switch (*keyword)
+    {
+      case 'C':
+      case 'c':
+      {
+        if (LocaleCompare((char *) keyword,"command") == 0)
           {
             char
-              path[MaxTextExtent];
+              *commands;
 
-            NTGhostscriptEXE(path,MaxTextExtent);
-            SubstituteString((char **) &commands,"@PSDelegate@",path);
-            SubstituteString((char **) &commands,"\\","/");
-          }
+            commands=AcquireString(token);
+#if defined(__WINDOWS__)
+            if (strchr(commands,'@') != (char *) NULL)
+              {
+                char
+                  path[MaxTextExtent];
+
+                NTGhostscriptEXE(path,MaxTextExtent);
+                SubstituteString((char **) &commands,"@PSDelegate@",path);
+                SubstituteString((char **) &commands,"\\","/");
+              }
 #endif
-        delegate_info->commands=commands;
+            SubstituteString((char **) &commands,"&amp;","\"");
+            SubstituteString((char **) &commands,"&quot;","\"");
+            SubstituteString((char **) &commands,"&gt;",">");
+            SubstituteString((char **) &commands,"&lt;","<");
+            delegate_info->commands=commands;
+            break;
+          }
+        break;
       }
-    attribute=GetXMLTreeAttribute(delegate,"decode");
-    if (attribute != (const char *) NULL)
+      case 'D':
+      case 'd':
       {
-        delegate_info->decode=ConstantString(attribute);
-        delegate_info->mode=1;
+        if (LocaleCompare((char *) keyword,"decode") == 0)
+          {
+            delegate_info->decode=ConstantString(token);
+            delegate_info->mode=1;
+            break;
+          }
+        break;
       }
-    attribute=GetXMLTreeAttribute(delegate,"encode");
-    if (attribute != (const char *) NULL)
+      case 'E':
+      case 'e':
       {
-        delegate_info->encode=ConstantString(attribute);
-        delegate_info->mode=(-1);
+        if (LocaleCompare((char *) keyword,"encode") == 0)
+          {
+            delegate_info->encode=ConstantString(token);
+            delegate_info->mode=(-1);
+            break;
+          }
+        break;
       }
-    attribute=GetXMLTreeAttribute(delegate,"mode");
-    if (attribute != (const char *) NULL)
+      case 'M':
+      case 'm':
       {
-        delegate_info->mode=1;
-        if (LocaleCompare(attribute,"bi") == 0)
-          delegate_info->mode=0;
-        if (LocaleCompare(attribute,"encode") == 0)
-          delegate_info->mode=(-1);
+        if (LocaleCompare((char *) keyword,"mode") == 0)
+          {
+            delegate_info->mode=1;
+            if (LocaleCompare(token,"bi") == 0)
+              delegate_info->mode=0;
+            else
+              if (LocaleCompare(token,"encode") == 0)
+                delegate_info->mode=(-1);
+            break;
+          }
+        break;
       }
-    attribute=GetXMLTreeAttribute(delegate,"spawn");
-    if (attribute != (const char *) NULL)
-      delegate_info->spawn=IsMagickTrue(attribute);
-    attribute=GetXMLTreeAttribute(delegate,"stealth");
-    if (attribute != (const char *) NULL)
-      delegate_info->stealth=IsMagickTrue(attribute);
-    attribute=GetXMLTreeAttribute(delegate,"thread-support");
-    if (attribute != (const char *) NULL)
-      delegate_info->thread_support=IsMagickTrue(attribute);
-    status=AppendValueToLinkedList(delegate_list,delegate_info);
-    if (status == MagickFalse)
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
-    delegate=GetNextXMLTreeTag(delegate);
+      case 'S':
+      case 's':
+      {
+        if (LocaleCompare((char *) keyword,"spawn") == 0)
+          {
+            delegate_info->spawn=IsMagickTrue(token);
+            break;
+          }
+        if (LocaleCompare((char *) keyword,"stealth") == 0)
+          {
+            delegate_info->stealth=IsMagickTrue(token);
+            break;
+          }
+        break;
+      }
+      case 'T':
+      case 't':
+      {
+        if (LocaleCompare((char *) keyword,"thread-support") == 0)
+          {
+            delegate_info->thread_support=IsMagickTrue(token);
+            break;
+          }
+        break;
+      }
+      default:
+        break;
+    }
   }
-  delegate_map=DestroyXMLTree(delegate_map);
+  token=(char *) RelinquishMagickMemory(token);
   return(status);
 }
 
@@ -1297,7 +1409,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: the font file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -1305,7 +1417,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
 static MagickBooleanType LoadDelegateLists(const char *filename,
   ExceptionInfo *exception)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   return(LoadDelegateList(DelegateMap,"built-in",0,exception));
 #else
   const StringInfo
