@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -38,8 +38,13 @@ static inline PixelPacket GetFillColor(const DrawInfo *draw_info,
   pattern=draw_info->fill_pattern;
   if (pattern == (Image *) NULL)
     return(draw_info->fill);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
   fill_color=AcquireOneVirtualPixel(pattern,TileVirtualPixelMethod,
     x+pattern->tile_offset.x,y+pattern->tile_offset.y,&pattern->exception);
+  if (pattern->matte == MagickFalse)
+    fill_color.opacity=OpaqueOpacity;
   return(fill_color);
 }
 
@@ -55,8 +60,13 @@ static inline PixelPacket GetStrokeColor(const DrawInfo *draw_info,
   pattern=draw_info->stroke_pattern;
   if (pattern == (Image *) NULL)
     return(draw_info->stroke);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
   stroke_color=AcquireOneVirtualPixel(pattern,TileVirtualPixelMethod,
     x+pattern->tile_offset.x,y+pattern->tile_offset.y,&pattern->exception);
+  if (pattern->matte == MagickFalse)
+    stroke_color.opacity=OpaqueOpacity;
   return(stroke_color);
 }
 
