@@ -10,14 +10,14 @@
 %        EEEEE   X  X   CCCC  EEEEE  P       T    IIIII   OOO   N   N         %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick Exception Methods                         %
+%                        MagickCore Exception Methods                         %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                                July 1993                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -106,10 +106,8 @@ MagickExport ExceptionInfo *AcquireExceptionInfo(void)
   exception=(ExceptionInfo *) AcquireMagickMemory(sizeof(*exception));
   if (exception == (ExceptionInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(exception,0,sizeof(*exception));
   GetExceptionInfo(exception);
   exception->relinquish=MagickTrue;
-  exception->signature=MagickSignature;
   return(exception);
 }
 
@@ -133,7 +131,7 @@ MagickExport ExceptionInfo *AcquireExceptionInfo(void)
 %
 %  A description of each parameter follows:
 %
-%    o exception: The exception info.
+%    o exception: the exception info.
 %
 */
 
@@ -194,7 +192,7 @@ MagickExport void ClearMagickException(ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o exception: The exception info.
+%    o exception: the exception info.
 %
 */
 MagickExport void CatchException(ExceptionInfo *exception)
@@ -365,11 +363,14 @@ static void DefaultWarningHandler(const ExceptionType magick_unused(severity),
 %
 %  A description of each parameter follows:
 %
-%    o exception: The exception info.
+%    o exception: the exception info.
 %
 */
 MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
 {
+  MagickBooleanType
+    relinquish;
+
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   AcquireSemaphoreInfo(&exception->semaphore);
@@ -377,10 +378,12 @@ MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
   if (exception->exceptions != (void *) NULL)
     exception->exceptions=(void *) DestroyLinkedList((LinkedListInfo *)
       exception->exceptions,DestroyExceptionElement);
-  exception->signature=(~MagickSignature);
-  RelinquishSemaphoreInfo(exception->semaphore);
-  exception->semaphore=DestroySemaphoreInfo(exception->semaphore);
+  relinquish=exception->relinquish;
   if (exception->relinquish != MagickFalse)
+    exception->signature=(~MagickSignature);
+  RelinquishSemaphoreInfo(exception->semaphore);
+  DestroySemaphoreInfo(&exception->semaphore);
+  if (relinquish != MagickFalse)
     exception=(ExceptionInfo *) RelinquishMagickMemory(exception);
   return(exception);
 }
@@ -404,7 +407,7 @@ MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o exception: The exception info.
+%    o exception: the exception info.
 %
 */
 MagickExport void GetExceptionInfo(ExceptionInfo *exception)
@@ -576,9 +579,9 @@ MagickExport const char *GetLocaleExceptionMessage(const ExceptionType severity,
 %
 %  A description of each parameter follows:
 %
-%    o exception: The exception info.
+%    o exception: the exception info.
 %
-%    o relative: The related exception info.
+%    o relative: the related exception info.
 %
 */
 MagickExport void InheritException(ExceptionInfo *exception,
@@ -695,7 +698,7 @@ MagickExport void MagickFatalError(const ExceptionType error,const char *reason,
 %
 %  A description of each parameter follows:
 %
-%    o warning: The warning severity.
+%    o warning: the warning severity.
 %
 %    o reason: Define the reason for the warning.
 %
@@ -729,7 +732,7 @@ MagickExport void MagickWarning(const ExceptionType warning,const char *reason,
 %
 %  A description of each parameter follows:
 %
-%    o handler: The method to handle errors.
+%    o handler: the method to handle errors.
 %
 */
 MagickExport ErrorHandler SetErrorHandler(ErrorHandler handler)
@@ -762,7 +765,7 @@ MagickExport ErrorHandler SetErrorHandler(ErrorHandler handler)
 %
 %  A description of each parameter follows:
 %
-%    o handler: The method to handle errors.
+%    o handler: the method to handle errors.
 %
 */
 MagickExport FatalErrorHandler SetFatalErrorHandler(FatalErrorHandler handler)
@@ -795,7 +798,7 @@ MagickExport FatalErrorHandler SetFatalErrorHandler(FatalErrorHandler handler)
 %
 %  A description of each parameter follows:
 %
-%    o handler: The method to handle warnings.
+%    o handler: the method to handle warnings.
 %
 */
 MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
@@ -830,13 +833,13 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 %
 %  A description of each parameter follows:
 %
-%    o exception: The exception info.
+%    o exception: the exception info.
 %
-%    o severity: The severity of the exception.
+%    o severity: the severity of the exception.
 %
-%    o tag: The message tag.
+%    o tag: the message tag.
 %
-%    o description: The exception description.
+%    o description: the exception description.
 %
 */
 MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
@@ -892,19 +895,19 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
 %
 %  A description of each parameter follows:
 %
-%    o exception: The exception info.
+%    o exception: the exception info.
 %
-%    o filename: The source module filename.
+%    o filename: the source module filename.
 %
-%    o function: The function name.
+%    o function: the function name.
 %
-%    o line: The line number of the source module.
+%    o line: the line number of the source module.
 %
 %    o severity: Specifies the numeric error category.
 %
-%    o tag: The locale tag.
+%    o tag: the locale tag.
 %
-%    o format: The output format.
+%    o format: the output format.
 %
 */
 
@@ -934,7 +937,7 @@ MagickExport MagickBooleanType ThrowMagickExceptionList(
   (void) CopyMagickString(reason,locale,MaxTextExtent);
   (void) ConcatenateMagickString(reason," ",MaxTextExtent);
   length=strlen(reason);
-#if defined(HAVE_VSNPRINTF)
+#if defined(MAGICKCORE_HAVE_VSNPRINTF)
   n=vsnprintf(reason+length,MaxTextExtent-length,format,operands);
 #else
   n=vsprintf(reason+length,format,operands);
