@@ -10,14 +10,14 @@
 %                             LLLLL   OOO    GGG                              %
 %                                                                             %
 %                                                                             %
-%                           Log ImageMagick Events                            %
+%                             MagickCore Log Events                           %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                September 2002                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -56,9 +56,12 @@
 #include "magick/utility.h"
 #include "magick/version.h"
 #include "magick/xml-tree.h"
-#if defined(HAVE_PTHREAD)
+#if defined(MAGICKCORE_HAVE_PTHREAD)
 #include <pthread.h>
 #endif
+#if defined(MAGICKCORE_HAVE_LINUX_UNISTD_H)
+#include <linux/unistd.h>
+#endif
 #if defined(__WINDOWS__)
 #include <windows.h>
 #endif
@@ -278,7 +281,7 @@ MagickExport void DestroyLogList(void)
     log_list=DestroyLinkedList(log_list,DestroyLogElement);
   instantiate_log=MagickFalse;
   RelinquishSemaphoreInfo(log_semaphore);
-  log_semaphore=DestroySemaphoreInfo(log_semaphore);
+  DestroySemaphoreInfo(&log_semaphore);
 }
 
 /*
@@ -301,7 +304,7 @@ MagickExport void DestroyLogList(void)
 %
 %  A description of each parameter follows:
 %
-%    o name: The log name.
+%    o name: the log name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -322,7 +325,7 @@ MagickExport const LogInfo *GetLogInfo(const char *name,
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
     return((const LogInfo *) GetValueFromLinkedList(log_list,0));
   /*
-    Search for named log.
+    Search for log tag.
   */
   AcquireSemaphoreInfo(&log_semaphore);
   ResetLinkedListIterator(log_list);
@@ -333,9 +336,9 @@ MagickExport const LogInfo *GetLogInfo(const char *name,
       break;
     p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   }
-  if (p == (LogInfo *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
-      "NoSuchElement","`%s'",name);
+  if (p != (LogInfo *) NULL)
+    (void) InsertValueInLinkedList(log_list,0,
+      RemoveElementByValueFromLinkedList(log_list,p));
   RelinquishSemaphoreInfo(log_semaphore);
   return(p);
 }
@@ -730,22 +733,22 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o type: The event type.
+%    o type: the event type.
 %
-%    o filename: The source module filename.
+%    o filename: the source module filename.
 %
-%    o function: The function name.
+%    o function: the function name.
 %
-%    o line: The line number of the source module.
+%    o line: the line number of the source module.
 %
-%    o format: The output format.
+%    o format: the output format.
 %
 */
 
 static unsigned long GetLogThreadId(void)
 {
-#if defined(HAVE_PTHREAD)
-  return((unsigned long) pthread_self());
+#if defined(MAGICKCORE_HAVE_PTHREAD) && defined(MAGICKCORE_HAVE_LINUX_UNISTD_H)
+  return((unsigned long) ((pid_t) syscall(__NR_gettid)));
 #endif
 #if defined(__WINDOWS__)
   return((unsigned long) GetCurrentThreadId());
@@ -1106,7 +1109,7 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
       return(MagickTrue);
     }
   domain=MagickOptionToMnemonic(MagickLogEventOptions,type);
-#if defined(HAVE_VSNPRINTF)
+#if defined(MAGICKCORE_HAVE_VSNPRINTF)
   n=vsnprintf(event,MaxTextExtent,format,operands);
 #else
   n=vsprintf(event,format,operands);
@@ -1163,8 +1166,8 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
               RelinquishSemaphoreInfo(log_semaphore);
               return(MagickFalse);
             }
-          log_info->append=IsAccessible(filename);
-          log_info->file=MagickOpenStream(filename,"ab");
+          log_info->append=IsPathAccessible(filename);
+          log_info->file=OpenMagickStream(filename,"ab");
           filename=(char  *) RelinquishMagickMemory(filename);
           if (log_info->file == (FILE *) NULL)
             {
@@ -1477,7 +1480,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The log configuration filename.
+%    o filename: the log configuration filename.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -1485,7 +1488,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 static MagickBooleanType LoadLogLists(const char *filename,
   ExceptionInfo *exception)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   return(LoadLogList(LogMap,"built-in",0,exception));
 #else
   const StringInfo
@@ -1639,7 +1642,7 @@ MagickExport LogEventType SetLogEventMask(const char *events)
 %
 %  A description of each parameter follows:
 %
-%    o format: The log record format.
+%    o format: the log record format.
 %
 */
 MagickExport void SetLogFormat(const char *format)
