@@ -10,7 +10,7 @@
 %           R  R    EEEEE   SSSSS  A   A  M   M  P      LLLLL  EEEEE          %
 %                                                                             %
 %                                                                             %
-%                           Pixel Resampling Methods.                         %
+%                      MagickCore Pixel Resampling Methods                    %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -18,7 +18,7 @@
 %                                August 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,6 +41,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/artifact.h"
 #include "magick/color-private.h"
 #include "magick/cache.h"
 #include "magick/draw.h"
@@ -57,7 +58,7 @@
 #include "magick/resize.h"
 #include "magick/resize-private.h"
 #include "magick/transform.h"
-#include "magick/signature.h"
+#include "magick/signature-private.h"
 /*
   Typedef declarations.
 */
@@ -112,12 +113,6 @@ struct _ResampleFilter
     signature;
 };
 
-/*
-  Forward declarations.
-*/
-static void
-  SetResampleFilter(ResampleFilter *,const FilterTypes,const double);
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -143,7 +138,7 @@ static void
 %  output pixel, the ResampleFilter structure generated holds that information
 %  between individual image resampling.
 %
-%  This function will make the appropriate OpenCacheView() calls
+%  This function will make the appropriate AcquireCacheView() calls
 %  to view the image, calling functions do not need to open a cache view.
 %
 %  Usage Example...
@@ -165,7 +160,7 @@ static void
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -190,7 +185,7 @@ MagickExport ResampleFilter *AcquireResampleFilter(const Image *image,
   (void) ResetMagickMemory(resample_filter,0,sizeof(*resample_filter));
 
   resample_filter->image=ReferenceImage((Image *) image);
-  resample_filter->view=OpenCacheView(resample_filter->image);
+  resample_filter->view=AcquireCacheView(resample_filter->image);
   resample_filter->exception=exception;
 
   resample_filter->debug=IsEventLogging();
@@ -246,7 +241,7 @@ MagickExport ResampleFilter *DestroyResampleFilter(
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       resample_filter->image->filename);
 
-  resample_filter->view=CloseCacheView(resample_filter->view);
+  resample_filter->view=DestroyCacheView(resample_filter->view);
   resample_filter->image=DestroyImage(resample_filter->image);
   resample_filter->signature=(~MagickSignature);
 
@@ -277,9 +272,9 @@ MagickExport ResampleFilter *DestroyResampleFilter(
 %
 %  A description of each parameter follows:
 %
-%    o resample_filter: The resample filter.
+%    o resample_filter: the resample filter.
 %
-%    o method: The pixel clor interpolation method.
+%    o method: the pixel clor interpolation method.
 %
 %    o x,y: A double representing the current (x,y) position of the pixel.
 %
@@ -468,10 +463,10 @@ static MagickPixelPacket InterpolateResampleFilter(
     default:
     {
       MagickPixelPacket
-        pixels[16];
+        pixels[4];
 
       MagickRealType
-        alpha[16],
+        alpha[4],
         gamma;
 
       PointInfo
@@ -484,20 +479,24 @@ static MagickPixelPacket InterpolateResampleFilter(
       indexes=AcquireCacheViewIndexes(resample_filter->view);
       for (i=0; i < 4L; i++)
       {
-        GetMagickPixelPacket(resample_filter->image,pixels+i);
-        SetMagickPixelPacket(resample_filter->image,p,indexes+i,pixels+i);
+        pixels[i].red=p[i].red;
+        pixels[i].green=p[i].green;
+        pixels[i].blue=p[i].blue;
+        pixels[i].opacity=p[i].opacity;
+        if (indexes != (IndexPacket *) NULL)
+          pixels[i].index=indexes[i];
         alpha[i]=1.0;
-        if (resample_filter->image->matte != MagickFalse)
-          {
-            alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
-            pixels[i].red*=alpha[i];
-            pixels[i].green*=alpha[i];
-            pixels[i].blue*=alpha[i];
-            if (resample_filter->image->colorspace == CMYKColorspace)
-              pixels[i].index*=alpha[i];
-          }
-        p++;
       }
+      if (resample_filter->image->matte != MagickFalse)
+        for (i=0; i < 4L; i++)
+        {
+          alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p[i].opacity);
+          pixels[i].red*=alpha[i];
+          pixels[i].green*=alpha[i];
+          pixels[i].blue*=alpha[i];
+          if (resample_filter->image->colorspace == CMYKColorspace)
+            pixels[i].index*=alpha[i];
+        }
       delta.x=x-floor(x);
       delta.y=y-floor(y);
       gamma=(((1.0-delta.y)*((1.0-delta.x)*alpha[0]+delta.x*alpha[1])+delta.y*
@@ -524,7 +523,7 @@ static MagickPixelPacket InterpolateResampleFilter(
     case FilterInterpolatePixel:
     {
       Image
-        *excert_image,
+        *excerpt_image,
         *filter_image;
 
       MagickPixelPacket
@@ -533,28 +532,31 @@ static MagickPixelPacket InterpolateResampleFilter(
       RectangleInfo
         geometry;
 
+      ViewInfo
+        *filter_view;
+
       geometry.width=4L;
       geometry.height=4L;
       geometry.x=(long) floor(x)-1L;
       geometry.y=(long) floor(y)-1L;
-      excert_image=ExcerptImage(resample_filter->image,&geometry,
+      excerpt_image=ExcerptImage(resample_filter->image,&geometry,
         resample_filter->exception);
-      if (excert_image == (Image *) NULL)
+      if (excerpt_image == (Image *) NULL)
         break;
-      filter_image=ResizeImage(excert_image,1,1,resample_filter->image->filter,
+      filter_image=ResizeImage(excerpt_image,1,1,resample_filter->image->filter,
         resample_filter->image->blur,resample_filter->exception);
-      excert_image=DestroyImage(excert_image);
+      excerpt_image=DestroyImage(excerpt_image);
       if (filter_image == (Image *) NULL)
         break;
-      p=AcquireImagePixels(filter_image,0,0,1,1,resample_filter->exception);
-      if (p == (const PixelPacket *) NULL)
+      filter_view=AcquireCacheView(filter_image);
+      p=AcquireCacheViewPixels(filter_view,0,0,1,1,resample_filter->exception);
+      if (p != (const PixelPacket *) NULL)
         {
-          filter_image=DestroyImage(filter_image);
-          break;
+          indexes=AcquireIndexes(filter_image);
+          GetMagickPixelPacket(resample_filter->image,pixels);
+          SetMagickPixelPacket(resample_filter->image,p,indexes,&pixel);
         }
-      indexes=AcquireIndexes(filter_image);
-      GetMagickPixelPacket(resample_filter->image,pixels);
-      SetMagickPixelPacket(resample_filter->image,p,indexes,&pixel);
+      filter_view=DestroyCacheView(filter_view);
       filter_image=DestroyImage(filter_image);
       break;
     }
@@ -804,7 +806,7 @@ static MagickPixelPacket InterpolateResampleFilter(
 %
 %  A description of each parameter follows:
 %
-%    o resample_filter: The Resampling Information Structure
+%    o resample_filter: the Resampling Information Structure
 %
 %    o u0,v0: A double representing the center of the area to resample,
 %             The distortion transformed transformed x,y coordinate.
@@ -825,8 +827,6 @@ MagickExport MagickPixelPacket ResamplePixelColor(
   assert(resample_filter != (ResampleFilter *) NULL);
   assert(resample_filter->signature == MagickSignature);
 
-  GetMagickPixelPacket(resample_filter->image,&pixel);
-
   if ( resample_filter->do_interpolate ) {
     pixel = InterpolateResampleFilter(resample_filter,
          resample_filter->interpolate,u0,v0);
@@ -835,8 +835,9 @@ MagickExport MagickPixelPacket ResamplePixelColor(
 
   /*
     Does resample area Miss the image?
-    And area is in virtual areas of solid color - return that color
+    And is that area a simple solid color - then return that color
   */
+  GetMagickPixelPacket(resample_filter->image,&pixel);
   hit = 0;
   switch ( resample_filter->virtual_pixel ) {
     case BackgroundVirtualPixelMethod:
@@ -867,7 +868,18 @@ MagickExport MagickPixelPacket ResamplePixelColor(
            )
         hit++;
       break;
-
+    case HorizontalTileVirtualPixelMethod:
+      if (    v0 + resample_filter->sqrtA < 0.0
+           || v0 - resample_filter->sqrtA > (double) resample_filter->image->rows
+           )
+        hit++;  /* outside the horizontally tiled images. */
+      break;
+    case VerticalTileVirtualPixelMethod:
+      if (    u0 + resample_filter->sqrtC < 0.0
+           || u0 - resample_filter->sqrtC > (double) resample_filter->image->columns
+           )
+        hit++;  /* outside the vertically tiled images. */
+      break;
     case DitherVirtualPixelMethod:
       if (    ( u0 + resample_filter->sqrtC < -32.0 && v0 + resample_filter->sqrtA < -32.0 )
            || ( u0 + resample_filter->sqrtC < -32.0
@@ -882,7 +894,7 @@ MagickExport MagickPixelPacket ResamplePixelColor(
     case TileVirtualPixelMethod:
     case MirrorVirtualPixelMethod:
     case RandomVirtualPixelMethod:
-      /* resampling of area always needed - no VP limits */
+      /* resampling of area is always needed - no VP limits */
       break;
   }
   if ( hit ) {
@@ -892,7 +904,7 @@ MagickExport MagickPixelPacket ResamplePixelColor(
   }
 
   /*
-    Scaling limits reached, return 'averaged' result.
+    Scaling limits reached, return an 'averaged' result.
   */
   if ( resample_filter->limit_reached ) {
     switch ( resample_filter->virtual_pixel ) {
@@ -925,6 +937,9 @@ MagickExport MagickPixelPacket ResamplePixelColor(
           Image
             *average_image;
 
+          ViewInfo
+            *average_view;
+
           GetMagickPixelPacket(resample_filter->image,
                 (MagickPixelPacket *)&(resample_filter->average_pixel));
           resample_filter->average_defined = MagickTrue;
@@ -934,15 +949,18 @@ MagickExport MagickPixelPacket ResamplePixelColor(
            resample_filter->exception);
           if (average_image == (Image *) NULL)
             return(resample_filter->average_pixel);
-          pixels=(PixelPacket *)AcquireImagePixels(average_image,0,0,1,1,
+          average_view=AcquireCacheView(average_image);
+          pixels=(PixelPacket *)AcquireCacheViewPixels(average_view,0,0,1,1,
             resample_filter->exception);
           if (pixels == (const PixelPacket *) NULL) {
+            average_view=DestroyCacheView(average_view);
             average_image=DestroyImage(average_image);
             return(resample_filter->average_pixel);
           }
-          indexes=(IndexPacket *) GetIndexes(average_image);
+          indexes=(IndexPacket *) GetCacheViewIndexes(average_view);
           SetMagickPixelPacket(resample_filter->image,pixels,indexes,
             &(resample_filter->average_pixel));
+          average_view=DestroyCacheView(average_view);
           average_image=DestroyImage(average_image);
         }
         pixel = resample_filter->average_pixel;
@@ -989,8 +1007,8 @@ MagickExport MagickPixelPacket ResamplePixelColor(
     DQ = resample_filter->A*(2.0*U+1) + resample_filter->B*V;
 
     /* get the scanline of pixels for this v */
-    pixels=AcquireCacheViewPixels(resample_filter->view,u,v,1UL*uw,1,
-      resample_filter->exception);
+    pixels=AcquireCacheViewPixels(resample_filter->view,u,v,(unsigned long) uw,
+      1,resample_filter->exception);
     if (pixels == (const PixelPacket *) NULL)
       return(pixel);
     indexes=AcquireCacheViewIndexes(resample_filter->view);
@@ -1059,12 +1077,13 @@ MagickExport MagickPixelPacket ResamplePixelColor(
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  ScaleResampleFilter() does all the calculations needed to resample an image
-%  at a specific scale, defined by two scaling vectors.  This is not using
+%  at a specific scale, defined by two scaling vectors.  This not using
 %  a orthogonal scaling, but two distorted scaling vectors.
 %
 %  The input vectors are produced by either finding the derivitives of
 %  the distortion function, or the partial derivitives from a distortion
-%  mapping.
+%  mapping.  They do not need to be the orthogonal derivatives, but can be
+%  caclated using radial/angluar derivatives.
 %
 %  If   u,v =  DistortEquation(x,y)
 %  Then the scaling vectors are the function derivitives...
@@ -1080,7 +1099,7 @@ MagickExport MagickPixelPacket ResamplePixelColor(
 %
 %  A description of each parameter follows:
 %
-%    o resample_filter: The resampling resample_filterrmation defining the
+%    o resample_filter: the resampling resample_filterrmation defining the
 %      image being resampled
 %
 %    o dux,duy,dvx,dvy:
@@ -1088,7 +1107,7 @@ MagickExport MagickPixelPacket ResamplePixelColor(
 %         for the X direction:  du/dx, dv/dx
 %         and the Y direction:  du/dy, dv/dy
 %         This defines the size and angle of the elliptical resampling area
-%         The order is the the order you generally would figure out the
+%         The order is the order you generally would figure out the
 %         deritives from the distortion equations.
 %
 */
@@ -1103,8 +1122,6 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
   resample_filter->limit_reached = MagickFalse;
   resample_filter->do_interpolate = MagickFalse;
 
-//printf("dux = %.5lf;  dvx = %.5lf;    duy = %.5lf;  dvy = %.5lf\n",dux,dvx,duy,dvy);
-
   /* A 'point' filter forces use of interpolation instead of area sampling */
   if ( resample_filter->filter == PointFilter ) {
     resample_filter->do_interpolate = MagickTrue;
@@ -1126,7 +1143,7 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
   F *= F;
 #define F_UNITY 1.0
 #else
-  /* This is Paul Heckbert's recomended "Higher Quality EWA" formula, from page
+  /* This Paul Heckbert's recomended "Higher Quality EWA" formula, from page
      60 in his thesis, which adds a unit circle to the elliptical area so are
      to do both Reconstruction and Prefiltering of the pixels in the
      resampling.  It also means it is likely to have at least 4 pixels within
@@ -1182,7 +1199,7 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
     Ellipse_Area = MagickPI*Major*Minor
     ellipse_angle =  arctan2(B, A-C);
 
-    This is actually used as part of parallelogram bounds
+    This actually used as part of parallelogram bounds
     max_horizontal_cross_section =  2*sqrt(F/A) = sqrt(C-B*B/A);
     max_vertical_cross_section   =  2*sqrt(F/C) = sqrt(A-B*B/C);
     */
@@ -1225,7 +1242,7 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   S e t R e s a m p l e F i l t e r                                         %
+%   S e t R e s a m p l e F i l t e r                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -1253,7 +1270,7 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
 %    o blur: filter blur factor (radial scaling) for elliptical weighting LUT
 %
 */
-static void SetResampleFilter(ResampleFilter *resample_filter,
+MagickExport void SetResampleFilter(ResampleFilter *resample_filter,
   const FilterTypes filter,const double blur)
 {
   register int
@@ -1278,32 +1295,31 @@ static void SetResampleFilter(ResampleFilter *resample_filter,
      being worked out.
 
      Future: Direct use of teh resize filters in "resize.c" to set the lookup
-     table.  This may require some scaling such as  x = sqrt(Q*PI/2048)
-     Note PI/2  is about 1.7 which seems to produce a better result.
+     table, based on the filters working support window.
   */
   r_scale = sqrt(1.0/(double)WLUT_WIDTH)/blur;
   r_scale *= 2; /* for 2 pixel radius of Improved Elliptical Formula */
 
   switch ( filter ) {
   case PointFilter:
-    /* This is equivelent to turning off the EWA algroithm.
+    /* This equivelent to turning off the EWA algroithm.
        Only Interpolated lookup will be used.  */
     break;
   default:
     /*
       Fill the LUT with a 1D resize filter function
-      But make teh Sinc/Bessel tapered window 2.0
+      But make the Sinc/Bessel tapered window 2.0
       I also normalize the result so the filter is 1.0
     */
     resize_filter = AcquireResizeFilter(resample_filter->image,filter,
          (MagickRealType)1.0,MagickTrue,resample_filter->exception);
     if (resize_filter != (ResizeFilter *) NULL) {
       resample_filter->support = GetResizeFilterSupport(resize_filter);
-      resample_filter->support /= blur;
+      resample_filter->support /= blur; /* taken into account above */
       resample_filter->support *= resample_filter->support;
       resample_filter->support *= (double)WLUT_WIDTH/4;
       if ( resample_filter->support >= (double)WLUT_WIDTH )
-           resample_filter->support = (double)WLUT_WIDTH;
+           resample_filter->support = (double)WLUT_WIDTH;  /* hack */
       for(Q=0; Q<WLUT_WIDTH; Q++)
         if ( (double) Q < resample_filter->support )
           resample_filter->filter_lut[Q] = (double)
@@ -1318,8 +1334,8 @@ static void SetResampleFilter(ResampleFilter *resample_filter,
            ModuleError, "UnableToSetFilteringValue",
            "Fall back to default EWA gaussian filter");
     }
-    /* FALLTHRU */
-  case GaussianFilter:
+    /* FALLTHRU - on exception */
+  /*case GaussianFilter:*/
   case UndefinedFilter:
     /*
       Create Normal Gaussian 2D Filter Weighted Lookup Table.
@@ -1336,17 +1352,16 @@ static void SetResampleFilter(ResampleFilter *resample_filter,
     resample_filter->support = WLUT_WIDTH;
     break;
   }
-#if 0
-  /* Debug output of the filter weighting LUT
-     Gnuplot the LUT with hoizontal adjusted to 'r' using...
-       plot [0:1][0:1] "lut.dat" using (sqrt($0/1024)):1 with lines
-     THe filter values is normalized for comparision
-  */
-  for(Q=0; Q<WLUT_WIDTH; Q++)
-    printf("%lf\n", resample_filter->filter_lut[Q] //);
-                      /resample_filter->filter_lut[0] );
-  exit(0);
-#endif
+  if (GetImageArtifact(resample_filter->image,"resample:verbose")
+        != (const char *) NULL)
+    /* Debug output of the filter weighting LUT
+      Gnuplot the LUT with hoizontal adjusted to 'r' using...
+        plot [0:2][-.2:1] "lut.dat" using (sqrt($0/1024)*2):1 with lines
+      THe filter values is normalized for comparision
+    */
+    for(Q=0; Q<WLUT_WIDTH; Q++)
+      printf("%lf\n", resample_filter->filter_lut[Q]
+                        /resample_filter->filter_lut[0] );
   return;
 }
 
@@ -1355,7 +1370,7 @@ static void SetResampleFilter(ResampleFilter *resample_filter,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S e t R e s a m p l e F i l t e r I n t e r p o l a t e M e t h o d        %
+%   S e t R e s a m p l e F i l t e r I n t e r p o l a t e M e t h o d       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -1371,9 +1386,9 @@ static void SetResampleFilter(ResampleFilter *resample_filter,
 %
 %  A description of each parameter follows:
 %
-%    o resample_filter: The resample filter.
+%    o resample_filter: the resample filter.
 %
-%    o method: The interpolation method.
+%    o method: the interpolation method.
 %
 */
 MagickExport MagickBooleanType SetResampleFilterInterpolateMethod(
@@ -1410,9 +1425,9 @@ MagickExport MagickBooleanType SetResampleFilterInterpolateMethod(
 %
 %  A description of each parameter follows:
 %
-%    o resample_filter: The resample filter.
+%    o resample_filter: the resample filter.
 %
-%    o method: The virtual pixel method.
+%    o method: the virtual pixel method.
 %
 */
 MagickExport MagickBooleanType SetResampleFilterVirtualPixelMethod(
