@@ -10,14 +10,14 @@
 %               SSSSS  EEEEE   GGGG  M   M  EEEEE  N   N    T                 %
 %                                                                             %
 %                                                                             %
-%        Methods to Segment an Image with Thresholding Fuzzy c-Means          %
+%    MagickCore Methods to Segment an Image with Thresholding Fuzzy c-Means   %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -91,6 +91,7 @@
 #include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantize.h"
 #include "magick/quantum.h"
 #include "magick/quantum-private.h"
@@ -217,7 +218,7 @@ static void
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o extrema:  Specifies a pointer to an array of integers.  They
 %      represent the peaks and valleys of the histogram for each color
@@ -260,7 +261,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     y;
 
   MagickBooleanType
-    status;
+    proceed;
 
   MagickRealType
     distance_squared,
@@ -390,14 +391,9 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           }
       p++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   /*
     Remove clusters that do not meet minimum cluster threshold.
@@ -491,7 +487,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Allocate image colormap.
   */
-  if (AllocateImageColormap(image,number_clusters) == MagickFalse)
+  if (AcquireImageColormap(image,number_clusters) == MagickFalse)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   cluster=head;
@@ -586,17 +582,12 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster)
   {
@@ -873,7 +864,7 @@ static void DerivativeHistogram(const MagickRealType *histogram,
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o cluster_threshold:  This MagickRealType represents the minimum number of
 %      pixels contained in a hexahedra before it can be considered valid
@@ -909,7 +900,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
     y;
 
   MagickBooleanType
-    status;
+    proceed;
 
   MagickPixelPacket
     pixel;
@@ -1071,14 +1062,9 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
           }
       p++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   /*
     Remove clusters that do not meet minimum cluster threshold.
@@ -1141,7 +1127,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
     (threshold+0.5));
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster)
   {
@@ -1626,7 +1612,7 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
     average_tau+=list[i]->tau;
   average_tau/=(MagickRealType) number_nodes;
   /*
-    Free resources.
+    Relinquish resources.
   */
   FreeNodes(root);
   zero_crossing=(ZeroCrossing *) RelinquishMagickMemory(zero_crossing);
@@ -1719,7 +1705,7 @@ static void ScaleSpace(const long *histogram,const MagickRealType tau,
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o colorspace: Indicate the colorspace.
 %
@@ -1792,7 +1778,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
   if (colorspace != RGBColorspace)
     (void) SetImageColorspace(image,colorspace);
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (i=0; i < MaxDimension; i++)
   {
