@@ -9,14 +9,14 @@
 %        SSSSS  IIIII   GGG   N   N  A   A    T     UUU   R  R   EEEEE        %
 %                                                                             %
 %                                                                             %
-%             Methods to Compute a Message Digest for an Image                %
+%         MagickCore Methods to Compute a Message Digest for an Image         %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,7 +48,44 @@
 #include "magick/quantum.h"
 #include "magick/quantum-private.h"
 #include "magick/signature.h"
+#include "magick/signature-private.h"
 #include "magick/string_.h"
+/*
+  Define declarations.
+*/
+#define SignatureBlocksize  64
+#define SignatureDigestsize  32
+
+/*
+  Typedef declarations.
+*/
+struct _SignatureInfo
+{   
+  unsigned int
+    digestsize,
+    blocksize;
+
+  StringInfo
+    *digest,
+    *message;
+
+  unsigned int
+    *accumulator,
+    low_order,
+    high_order;
+
+  size_t
+    offset;
+
+  MagickBooleanType
+    lsb_first;
+
+  long
+    timestamp;
+
+  unsigned long
+    signature;
+};
 
 /*
   Forward declarations.
@@ -61,60 +98,46 @@ static void
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   F i n a l i z e S i g n a t u r e                                         %
++   A c q u i r e S i g n a t u r e I n f o                                   %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  FinalizeSignature() finalizes the SHA message digest computation.
+%  AcquireSignatureInfo() allocate the SignatureInfo structure.
 %
-%  The format of the FinalizeSignature method is:
+%  The format of the AcquireSignatureInfo method is:
 %
-%      FinalizeSignature(SignatureInfo *signature_info)
-%
-%  A description of each parameter follows:
-%
-%    o signature_info: The address of a structure of type SignatureInfo.
+%      SignatureInfo *AcquireSignatureInfo(void)
 %
 */
-MagickExport void FinalizeSignature(SignatureInfo *signature_info)
+MagickExport SignatureInfo *AcquireSignatureInfo(void)
 {
-  long
-    count;
+  SignatureInfo
+    *signature_info;
 
-  unsigned long
-    high_order,
-    low_order;
+  unsigned int
+    lsb_first;
 
-  /*
-    Add padding and return the message digest.
-  */
-  assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
-  low_order=signature_info->low_order;
-  high_order=signature_info->high_order;
-  count=(long) ((low_order >> 3) & 0x3f);
-  signature_info->message[count++]=(unsigned char) 0x80;
-  if (count <= (MagickSignatureSize-8))
-    (void) ResetMagickMemory(signature_info->message+count,0,(size_t)
-      (MagickSignatureSize-8-count));
-  else
-    {
-      (void) ResetMagickMemory(signature_info->message+count,0,(size_t)
-        (MagickSignatureSize-count));
-      TransformSignature(signature_info);
-      (void) ResetMagickMemory(signature_info->message,0,MagickSignatureSize-8);
-    }
-  signature_info->message[56]=(unsigned char) (high_order >> 24);
-  signature_info->message[57]=(unsigned char) (high_order >> 16);
-  signature_info->message[58]=(unsigned char) (high_order >> 8);
-  signature_info->message[59]=(unsigned char) high_order;
-  signature_info->message[60]=(unsigned char) (low_order >> 24);
-  signature_info->message[61]=(unsigned char) (low_order >> 16);
-  signature_info->message[62]=(unsigned char) (low_order >> 8);
-  signature_info->message[63]=(unsigned char) low_order;
-  TransformSignature(signature_info);
+  signature_info=(SignatureInfo *) AcquireMagickMemory(sizeof(*signature_info));
+  if (signature_info == (SignatureInfo *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) ResetMagickMemory(signature_info,0,sizeof(*signature_info));
+  signature_info->digestsize=SignatureDigestsize;
+  signature_info->blocksize=SignatureBlocksize;
+  signature_info->digest=AcquireStringInfo(SignatureDigestsize);
+  signature_info->message=AcquireStringInfo(SignatureBlocksize);
+  signature_info->accumulator=(unsigned int *) AcquireQuantumMemory(
+    SignatureBlocksize,sizeof(*signature_info->accumulator));
+  if (signature_info->accumulator == (unsigned int *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  lsb_first=1;
+  signature_info->lsb_first=(int) (*(char *) &lsb_first) == 1 ? MagickTrue :
+    MagickFalse;
+  signature_info->timestamp=(long) time(0);
+  signature_info->signature=MagickSignature;
+  InitializeSignature(signature_info);
+  return(signature_info);
 }
 
 /*
@@ -122,42 +145,298 @@ MagickExport void FinalizeSignature(SignatureInfo *signature_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   G e t S i g n a t u r e I n f o                                           %
++   D e s t r o y S i g n a t u r e I n f o                                   %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetSignatureInfo() initializes the SHA message digest structure.
+%  DestroySignatureInfo() zeros memory associated with the SignatureInfo
+%  structure.
 %
-%  The format of the GetSignatureInfo method is:
+%  The format of the DestroySignatureInfo method is:
 %
-%      GetSignatureInfo(SignatureInfo *signature_info)
+%      SignatureInfo *DestroySignatureInfo(SignatureInfo *signature_info)
 %
 %  A description of each parameter follows:
 %
-%    o signature_info: The address of a structure of type SignatureInfo.
+%    o signature_info: the cipher signature_info.
 %
 */
-MagickExport void GetSignatureInfo(SignatureInfo *signature_info)
+MagickExport SignatureInfo *DestroySignatureInfo(SignatureInfo *signature_info)
 {
-  unsigned long
-    lsb_first;
-
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(signature_info != (SignatureInfo *) NULL);
-  (void) ResetMagickMemory(signature_info,0,sizeof(*signature_info));
-  signature_info->digest[0]=0x6a09e667UL;
-  signature_info->digest[1]=0xbb67ae85UL;
-  signature_info->digest[2]=0x3c6ef372UL;
-  signature_info->digest[3]=0xa54ff53aUL;
-  signature_info->digest[4]=0x510e527fUL;
-  signature_info->digest[5]=0x9b05688cUL;
-  signature_info->digest[6]=0x1f83d9abUL;
-  signature_info->digest[7]=0x5be0cd19UL;
-  lsb_first=1;
-  signature_info->lsb_first=
-    (*(char *) &lsb_first) == 1 ? MagickTrue : MagickFalse;
-  signature_info->signature=MagickSignature;
+  assert(signature_info->signature == MagickSignature);
+  if (signature_info->accumulator != (unsigned int *) NULL)
+    signature_info->accumulator=(unsigned int *) RelinquishMagickMemory(
+      signature_info->accumulator);
+  if (signature_info->message != (StringInfo *) NULL)
+    signature_info->message=DestroyStringInfo(signature_info->message);
+  if (signature_info->digest != (StringInfo *) NULL)
+    signature_info->digest=DestroyStringInfo(signature_info->digest);
+  signature_info->signature=(~MagickSignature);
+  signature_info=(SignatureInfo *) RelinquishMagickMemory(signature_info);
+  return(signature_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   F i n a l i z e S i g n a t u r e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  FinalizeSignature() finalizes the Signature message accumulator computation.
+%
+%  The format of the FinalizeSignature method is:
+%
+%      FinalizeSignature(SignatureInfo *signature_info)
+%
+%  A description of each parameter follows:
+%
+%    o signature_info: the address of a structure of type SignatureInfo.
+%
+*/
+MagickExport void FinalizeSignature(SignatureInfo *signature_info)
+{
+  register long
+    i;
+
+  register unsigned char
+    *q;
+
+  register unsigned int
+    *p;
+
+  unsigned char
+    *datum;
+
+  unsigned int
+    count,
+    high_order,
+    low_order;
+
+  /*
+    Add padding and return the message accumulator.
+  */
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
+  low_order=signature_info->low_order;
+  high_order=signature_info->high_order;
+  count=((low_order >> 3) & 0x3f);
+  datum=GetStringInfoDatum(signature_info->message);
+  datum[count++]=(unsigned char) 0x80;
+  if (count <= (unsigned int) (GetStringInfoLength(signature_info->message)-8))
+    (void) ResetMagickMemory(datum+count,0,GetStringInfoLength(
+      signature_info->message)-8-count);
+  else
+    {
+      (void) ResetMagickMemory(datum+count,0,GetStringInfoLength(
+        signature_info->message)-count);
+      TransformSignature(signature_info);
+      (void) ResetMagickMemory(datum,0,GetStringInfoLength(
+        signature_info->message)-8);
+    }
+  datum[56]=(unsigned char) (high_order >> 24);
+  datum[57]=(unsigned char) (high_order >> 16);
+  datum[58]=(unsigned char) (high_order >> 8);
+  datum[59]=(unsigned char) high_order;
+  datum[60]=(unsigned char) (low_order >> 24);
+  datum[61]=(unsigned char) (low_order >> 16);
+  datum[62]=(unsigned char) (low_order >> 8);
+  datum[63]=(unsigned char) low_order;
+  TransformSignature(signature_info);
+  p=signature_info->accumulator;
+  q=GetStringInfoDatum(signature_info->digest);
+  for (i=0; i < (SignatureDigestsize/4); i++)
+  {
+    *q++=(unsigned char) ((*p >> 24) & 0xff);
+    *q++=(unsigned char) ((*p >> 16) & 0xff);
+    *q++=(unsigned char) ((*p >> 8) & 0xff);
+    *q++=(unsigned char) (*p & 0xff);
+    p++;
+  }
+  /*
+    Reset working registers.
+  */
+  count=0;
+  high_order=0;
+  low_order=0;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   G e t S i g n a t u r e B l o c k s i z e                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetSignatureBlocksize() returns the Signature blocksize.
+%
+%  The format of the GetSignatureBlocksize method is:
+%
+%      unsigned int *GetSignatureBlocksize(const SignatureInfo *signature_info)
+%
+%  A description of each parameter follows:
+%
+%    o signature_info: the signature info.
+%
+*/
+MagickExport unsigned int GetSignatureBlocksize(
+  const SignatureInfo *signature_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
+  return(signature_info->blocksize);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   G e t S i g n a t u r e D i g e s t                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetSignatureDigest() returns the signature digest.
+%
+%  The format of the GetSignatureDigest method is:
+%
+%      const StringInfo *GetSignatureDigest(const SignatureInfo *signature_info)
+%
+%  A description of each parameter follows:
+%
+%    o signature_info: the signature info.
+%
+*/
+MagickExport const StringInfo *GetSignatureDigest(
+  const SignatureInfo *signature_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
+  return(signature_info->digest);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   G e t S i g n a t u r e D i g e s t s i z e                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetSignatureDigestsize() returns the Signature digest size.
+%
+%  The format of the GetSignatureDigestsize method is:
+%
+%      unsigned int *GetSignatureDigestsize(const SignatureInfo *signature_info)
+%
+%  A description of each parameter follows:
+%
+%    o signature_info: the signature info.
+%
+*/
+MagickExport unsigned int GetSignatureDigestsize(
+  const SignatureInfo *signature_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
+  return(signature_info->digestsize);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   I n i t i a l i z e S i g n a t u r e                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IntializeSignature() intializes the Signature accumulator.
+%
+%  The format of the DestroySignatureInfo method is:
+%
+%      void InitializeSignatureInfo(SignatureInfo *signature_info)
+%
+%  A description of each parameter follows:
+%
+%    o signature_info: the cipher signature_info.
+%
+*/
+MagickExport void InitializeSignature(SignatureInfo *signature_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
+  signature_info->accumulator[0]=0x6a09e667U;
+  signature_info->accumulator[1]=0xbb67ae85U;
+  signature_info->accumulator[2]=0x3c6ef372U;
+  signature_info->accumulator[3]=0xa54ff53aU;
+  signature_info->accumulator[4]=0x510e527fU;
+  signature_info->accumulator[5]=0x9b05688cU;
+  signature_info->accumulator[6]=0x1f83d9abU;
+  signature_info->accumulator[7]=0x5be0cd19U;
+  signature_info->low_order=0;
+  signature_info->high_order=0;
+  signature_info->offset=0;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   S e t S i g n a t u r e D i g e s t                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetSignatureDigest() set the signature digest.
+%
+%  The format of the SetSignatureDigest method is:
+%
+%      SetSignatureDigest(SignatureInfo *signature_info,
+%        const StringInfo *digest)
+%
+%  A description of each parameter follows:
+%
+%    o signature_info: the signature info.
+%
+%    o digest: the digest.
+%
+*/
+MagickExport void SetSignatureDigest(SignatureInfo *signature_info,
+  const StringInfo *digest)
+{
+  /*
+    Set the signature accumulator.
+  */
+  assert(signature_info != (SignatureInfo *) NULL);
+  assert(signature_info->signature == MagickSignature);
+  SetStringInfo(signature_info->digest,digest);
 }
 
 /*
@@ -182,113 +461,81 @@ MagickExport void GetSignatureInfo(SignatureInfo *signature_info)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport MagickBooleanType SignatureImage(Image *image)
 {
   char
-    signature[MaxTextExtent];
-
-  const IndexPacket
-    *indexes;
+    *hex_signature;
 
   long
     y;
 
+  QuantumInfo
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
+
   register const PixelPacket
     *p;
 
-  register long
-    x;
-
-  register unsigned char
-    *q;
-
   SignatureInfo
-    signature_info;
+    *signature_info;
 
   size_t
     length;
 
-  unsigned char
-    *message;
+  StringInfo
+    *signature;
 
-  unsigned long
-    quantum;
+  unsigned char
+    *pixels;
 
   /*
-    Allocate memory for digital signature.
+    Compute image digital signature.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  length=(size_t) image->columns;
-  message=(unsigned char *) AcquireQuantumMemory(length,20UL*sizeof(*message));
-  if (message == (unsigned char *) NULL)
+  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
+  if (quantum_info == (QuantumInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  /*
-    Compute image digital signature.
-  */
-  GetSignatureInfo(&signature_info);
+  quantum_type=RGBQuantum;
+  if (image->matte != MagickFalse)
+    quantum_type=RGBAQuantum;
+  if (image->colorspace == CMYKColorspace)
+    {
+      quantum_type=CMYKQuantum;
+      if (image->matte != MagickFalse)
+        quantum_type=CMYKAQuantum;
+    }
+  signature_info=AcquireSignatureInfo();
+  signature=AcquireStringInfo(quantum_info->extent);
+  pixels=GetQuantumPixels(quantum_info);
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=AcquireIndexes(image);
-    q=message;
-    for (x=0; x < (long) image->columns; x++)
-    {
-      quantum=ScaleQuantumToLong(p->red);
-      *q++=(unsigned char) (quantum >> 24);
-      *q++=(unsigned char) (quantum >> 16);
-      *q++=(unsigned char) (quantum >> 8);
-      *q++=(unsigned char) quantum;
-      quantum=ScaleQuantumToLong(p->green);
-      *q++=(unsigned char) (quantum >> 24);
-      *q++=(unsigned char) (quantum >> 16);
-      *q++=(unsigned char) (quantum >> 8);
-      *q++=(unsigned char) quantum;
-      quantum=ScaleQuantumToLong(p->blue);
-      *q++=(unsigned char) (quantum >> 24);
-      *q++=(unsigned char) (quantum >> 16);
-      *q++=(unsigned char) (quantum >> 8);
-      *q++=(unsigned char) quantum;
-      if (image->colorspace == CMYKColorspace)
-        {
-          quantum=ScaleQuantumToLong(indexes[x]);
-          *q++=(unsigned char) (quantum >> 24);
-          *q++=(unsigned char) (quantum >> 16);
-          *q++=(unsigned char) (quantum >> 8);
-          *q++=(unsigned char) quantum;
-        }
-      if (image->matte != MagickFalse)
-        {
-          quantum=ScaleQuantumToLong(p->opacity);
-          *q++=(unsigned char) (quantum >> 24);
-          *q++=(unsigned char) (quantum >> 16);
-          *q++=(unsigned char) (quantum >> 8);
-          *q++=(unsigned char) quantum;
-        }
-      p++;
-    }
-    UpdateSignature(&signature_info,message,(size_t) (q-message));
+    length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
+    SetStringInfoLength(signature,length);
+    SetStringInfoDatum(signature,pixels);
+    UpdateSignature(signature_info,signature);
   }
-  FinalizeSignature(&signature_info);
-  message=(unsigned char *) RelinquishMagickMemory(message);
-  /*
-    Convert digital signature to a 64 character hex string.
-  */
-  (void) FormatMagickString(signature,MaxTextExtent,
-    "%08lx%08lx%08lx%08lx%08lx%08lx%08lx%08lx",signature_info.digest[0],
-    signature_info.digest[1],signature_info.digest[2],signature_info.digest[3],
-    signature_info.digest[4],signature_info.digest[5],signature_info.digest[6],
-    signature_info.digest[7]);
+  FinalizeSignature(signature_info);
+  hex_signature=StringInfoToHexString(GetSignatureDigest(signature_info));
   (void) DeleteImageProperty(image,"signature");
-  (void) SetImageProperty(image,"signature",signature);
+  (void) SetImageProperty(image,"signature",hex_signature);
+  /*
+    Free resources.
+  */
+  hex_signature=DestroyString(hex_signature);
+  signature=DestroyStringInfo(signature);
+  signature_info=DestroySignatureInfo(signature_info);
   return(MagickTrue);
 }
 
@@ -303,7 +550,7 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  TransformSignature() transforms the SHA message digest.
+%  TransformSignature() transforms the Signature message accumulator.
 %
 %  The format of the TransformSignature method is:
 %
@@ -311,26 +558,26 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o signature_info: The address of a structure of type SignatureInfo.
+%    o signature_info: the address of a structure of type SignatureInfo.
 %
 */
 
-static inline unsigned long Ch(unsigned long x,unsigned long y,unsigned long z)
+static inline unsigned int Ch(unsigned int x,unsigned int y,unsigned int z)
 {
   return((x & y) ^ (~x & z));
 }
 
-static inline unsigned long Maj(unsigned long x,unsigned long y,unsigned long z)
+static inline unsigned int Maj(unsigned int x,unsigned int y,unsigned int z)
 {
   return((x & y) ^ (x & z) ^ (y & z));
 }
 
-static inline unsigned long Trunc32(unsigned long x)
+static inline unsigned int Trunc32(unsigned int x)
 {
-  return((unsigned long) (x & 0xffffffffUL));
+  return((unsigned int) (x & 0xffffffffU));
 }
 
-static unsigned long RotateRight(unsigned long x,unsigned long n)
+static unsigned int RotateRight(unsigned int x,unsigned int n)
 {
   return(Trunc32((x >> n) | (x << (32-n))));
 }
@@ -351,25 +598,25 @@ static void TransformSignature(SignatureInfo *signature_info)
   register unsigned char
     *p;
 
-  static unsigned long
+  static unsigned int
     K[64] =
     {
-      0x428a2f98UL, 0x71374491UL, 0xb5c0fbcfUL, 0xe9b5dba5UL, 0x3956c25bUL,
-      0x59f111f1UL, 0x923f82a4UL, 0xab1c5ed5UL, 0xd807aa98UL, 0x12835b01UL,
-      0x243185beUL, 0x550c7dc3UL, 0x72be5d74UL, 0x80deb1feUL, 0x9bdc06a7UL,
-      0xc19bf174UL, 0xe49b69c1UL, 0xefbe4786UL, 0x0fc19dc6UL, 0x240ca1ccUL,
-      0x2de92c6fUL, 0x4a7484aaUL, 0x5cb0a9dcUL, 0x76f988daUL, 0x983e5152UL,
-      0xa831c66dUL, 0xb00327c8UL, 0xbf597fc7UL, 0xc6e00bf3UL, 0xd5a79147UL,
-      0x06ca6351UL, 0x14292967UL, 0x27b70a85UL, 0x2e1b2138UL, 0x4d2c6dfcUL,
-      0x53380d13UL, 0x650a7354UL, 0x766a0abbUL, 0x81c2c92eUL, 0x92722c85UL,
-      0xa2bfe8a1UL, 0xa81a664bUL, 0xc24b8b70UL, 0xc76c51a3UL, 0xd192e819UL,
-      0xd6990624UL, 0xf40e3585UL, 0x106aa070UL, 0x19a4c116UL, 0x1e376c08UL,
-      0x2748774cUL, 0x34b0bcb5UL, 0x391c0cb3UL, 0x4ed8aa4aUL, 0x5b9cca4fUL,
-      0x682e6ff3UL, 0x748f82eeUL, 0x78a5636fUL, 0x84c87814UL, 0x8cc70208UL,
-      0x90befffaUL, 0xa4506cebUL, 0xbef9a3f7UL, 0xc67178f2UL
+      0x428a2f98U, 0x71374491U, 0xb5c0fbcfU, 0xe9b5dba5U, 0x3956c25bU,
+      0x59f111f1U, 0x923f82a4U, 0xab1c5ed5U, 0xd807aa98U, 0x12835b01U,
+      0x243185beU, 0x550c7dc3U, 0x72be5d74U, 0x80deb1feU, 0x9bdc06a7U,
+      0xc19bf174U, 0xe49b69c1U, 0xefbe4786U, 0x0fc19dc6U, 0x240ca1ccU,
+      0x2de92c6fU, 0x4a7484aaU, 0x5cb0a9dcU, 0x76f988daU, 0x983e5152U,
+      0xa831c66dU, 0xb00327c8U, 0xbf597fc7U, 0xc6e00bf3U, 0xd5a79147U,
+      0x06ca6351U, 0x14292967U, 0x27b70a85U, 0x2e1b2138U, 0x4d2c6dfcU,
+      0x53380d13U, 0x650a7354U, 0x766a0abbU, 0x81c2c92eU, 0x92722c85U,
+      0xa2bfe8a1U, 0xa81a664bU, 0xc24b8b70U, 0xc76c51a3U, 0xd192e819U,
+      0xd6990624U, 0xf40e3585U, 0x106aa070U, 0x19a4c116U, 0x1e376c08U,
+      0x2748774cU, 0x34b0bcb5U, 0x391c0cb3U, 0x4ed8aa4aU, 0x5b9cca4fU,
+      0x682e6ff3U, 0x748f82eeU, 0x78a5636fU, 0x84c87814U, 0x8cc70208U,
+      0x90befffaU, 0xa4506cebU, 0xbef9a3f7U, 0xc67178f2U
     };  /* 32-bit fractional part of the cube root of the first 64 primes */
 
-  unsigned long
+  unsigned int
     A,
     B,
     C,
@@ -385,30 +632,30 @@ static void TransformSignature(SignatureInfo *signature_info)
     W[64];
 
   shift=32;
-  p=signature_info->message;
+  p=GetStringInfoDatum(signature_info->message);
   if (signature_info->lsb_first == MagickFalse)
     {
-      if (sizeof(unsigned long) <= 4)
+      if (sizeof(unsigned int) <= 4)
         for (i=0; i < 16; i++)
         {
-          T=(*((unsigned long *) p));
+          T=(*((unsigned int *) p));
           p+=4;
           W[i]=Trunc32(T);
         }
       else
         for (i=0; i < 16; i+=2)
         {
-          T=(*((unsigned long *) p));
+          T=(*((unsigned int *) p));
           p+=8;
           W[i]=Trunc32(T >> shift);
           W[i+1]=Trunc32(T);
         }
     }
   else
-    if (sizeof(unsigned long) <= 4)
+    if (sizeof(unsigned int) <= 4)
       for (i=0; i < 16; i++)
       {
-        T=(*((unsigned long *) p));
+        T=(*((unsigned int *) p));
         p+=4;
         W[i]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
           ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
@@ -416,7 +663,7 @@ static void TransformSignature(SignatureInfo *signature_info)
     else
       for (i=0; i < 16; i+=2)
       {
-        T=(*((unsigned long *) p));
+        T=(*((unsigned int *) p));
         p+=8;
         W[i]=((T << 24) & 0xff000000) | ((T << 8) & 0x00ff0000) |
           ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
@@ -425,16 +672,16 @@ static void TransformSignature(SignatureInfo *signature_info)
           ((T >> 8) & 0x0000ff00) | ((T >> 24) & 0x000000ff);
       }
   /*
-    Copy digest to registers.
+    Copy accumulator to registers.
   */
-  A=signature_info->digest[0];
-  B=signature_info->digest[1];
-  C=signature_info->digest[2];
-  D=signature_info->digest[3];
-  E=signature_info->digest[4];
-  F=signature_info->digest[5];
-  G=signature_info->digest[6];
-  H=signature_info->digest[7];
+  A=signature_info->accumulator[0];
+  B=signature_info->accumulator[1];
+  C=signature_info->accumulator[2];
+  D=signature_info->accumulator[3];
+  E=signature_info->accumulator[4];
+  F=signature_info->accumulator[5];
+  G=signature_info->accumulator[6];
+  H=signature_info->accumulator[7];
   for (i=16; i < 64; i++)
     W[i]=Trunc32(Sigma1(W[i-2])+W[i-7]+Sigma0(W[i-15])+W[i-16]);
   for (j=0; j < 64; j++)
@@ -451,16 +698,31 @@ static void TransformSignature(SignatureInfo *signature_info)
     A=Trunc32(T1+T2);
   }
   /*
-    Add registers back to digest.
+    Add registers back to accumulator.
   */
-  signature_info->digest[0]=Trunc32(signature_info->digest[0]+A);
-  signature_info->digest[1]=Trunc32(signature_info->digest[1]+B);
-  signature_info->digest[2]=Trunc32(signature_info->digest[2]+C);
-  signature_info->digest[3]=Trunc32(signature_info->digest[3]+D);
-  signature_info->digest[4]=Trunc32(signature_info->digest[4]+E);
-  signature_info->digest[5]=Trunc32(signature_info->digest[5]+F);
-  signature_info->digest[6]=Trunc32(signature_info->digest[6]+G);
-  signature_info->digest[7]=Trunc32(signature_info->digest[7]+H);
+  signature_info->accumulator[0]=Trunc32(signature_info->accumulator[0]+A);
+  signature_info->accumulator[1]=Trunc32(signature_info->accumulator[1]+B);
+  signature_info->accumulator[2]=Trunc32(signature_info->accumulator[2]+C);
+  signature_info->accumulator[3]=Trunc32(signature_info->accumulator[3]+D);
+  signature_info->accumulator[4]=Trunc32(signature_info->accumulator[4]+E);
+  signature_info->accumulator[5]=Trunc32(signature_info->accumulator[5]+F);
+  signature_info->accumulator[6]=Trunc32(signature_info->accumulator[6]+G);
+  signature_info->accumulator[7]=Trunc32(signature_info->accumulator[7]+H);
+  /*
+    Reset working registers.
+  */
+  A=0;
+  B=0;
+  C=0;
+  D=0;
+  E=0;
+  F=0;
+  G=0;
+  H=0;
+  T=0;
+  T1=0;
+  T2=0;
+  (void) ResetMagickMemory(W,0,sizeof(W));
 }
 
 /*
@@ -474,65 +736,74 @@ static void TransformSignature(SignatureInfo *signature_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  UpdateSignature() updates the SHA message digest.
+%  UpdateSignature() updates the Signature message accumulator.
 %
 %  The format of the UpdateSignature method is:
 %
-%      UpdateSignature(SignatureInfo *signature_info,
-%        const unsigned char *message,const size_t length)
+%      UpdateSignature(SignatureInfo *signature_info,const StringInfo *message)
 %
 %  A description of each parameter follows:
 %
-%    o signature_info: The address of a structure of type SignatureInfo.
+%    o signature_info: the address of a structure of type SignatureInfo.
 %
-%    o message: the message
-%
-%    o length: The length of the message.
+%    o message: the message.
 %
 */
 MagickExport void UpdateSignature(SignatureInfo *signature_info,
-  const unsigned char *message,const size_t length)
+  const StringInfo *message)
 {
-  register long
+  register size_t
     i;
 
-  unsigned long
-    count,
+  register unsigned char
+    *p;
+
+  size_t
     n;
 
+  unsigned int
+    length;
+
   /*
-    Update the SHA digest.
+    Update the Signature accumulator.
   */
   assert(signature_info != (SignatureInfo *) NULL);
   assert(signature_info->signature == MagickSignature);
-  n=(unsigned long) length;
-  count=Trunc32(signature_info->low_order+(n << 3));
-  if (count < signature_info->low_order)
+  n=GetStringInfoLength(message);
+  length=Trunc32((unsigned int) (signature_info->low_order+(n << 3)));
+  if (length < signature_info->low_order)
     signature_info->high_order++;
-  signature_info->low_order=count;
-  signature_info->high_order+=(n >> 29);
+  signature_info->low_order=length;
+  signature_info->high_order+=(unsigned int) (n >> 29);
+  p=GetStringInfoDatum(message);
   if (signature_info->offset != 0)
     {
-      i=MagickSignatureSize-signature_info->offset;
-      if (i > (long) n)
-        i=(long) n;
-      (void) CopyMagickMemory(signature_info->message+signature_info->offset,
-        message,(size_t) i);
+      i=GetStringInfoLength(signature_info->message)-signature_info->offset;
+      if (i > n)
+        i=n;
+      (void) memcpy(GetStringInfoDatum(signature_info->message)+
+        signature_info->offset,p,i);
       n-=i;
-      message+=i;
+      p+=i;
       signature_info->offset+=i;
-      if (signature_info->offset != MagickSignatureSize)
+      if (signature_info->offset !=
+          GetStringInfoLength(signature_info->message))
         return;
       TransformSignature(signature_info);
     }
-  while (n >= MagickSignatureSize)
+  while (n >= GetStringInfoLength(signature_info->message))
   {
-    (void) CopyMagickMemory(signature_info->message,message,
-      MagickSignatureSize);
-    message+=MagickSignatureSize;
-    n-=MagickSignatureSize;
+    SetStringInfoDatum(signature_info->message,p);
+    p+=GetStringInfoLength(signature_info->message);
+    n-=GetStringInfoLength(signature_info->message);
     TransformSignature(signature_info);
   }
-  (void) CopyMagickMemory(signature_info->message,message,(size_t) n);
-  signature_info->offset=(long) n;
+  (void) memcpy(GetStringInfoDatum(signature_info->message),p,n);
+  signature_info->offset=n;
+  /*
+    Reset working registers.
+  */
+  i=0;
+  n=0;
+  length=0;
 }
