@@ -10,14 +10,14 @@
 %                  SSSSS     T    R  R   IIIII  N   N   GGGG                  %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick String Methods                            %
+%                        MagickCore String Methods                            %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,13 +49,13 @@
 #include "magick/memory_.h"
 #include "magick/property.h"
 #include "magick/resource_.h"
-#include "magick/signature.h"
+#include "magick/signature-private.h"
 #include "magick/string_.h"
 
 /*
   Static declarations.
 */
-#if !defined(HAVE_STRCASECMP) || !defined(HAVE_STRNCASECMP)
+#if !defined(MAGICKCORE_HAVE_STRCASECMP) || !defined(MAGICKCORE_HAVE_STRNCASECMP)
 static const unsigned char
   AsciiMap[] =
   {
@@ -149,7 +149,7 @@ MagickExport char *AcquireString(const char *source)
 %
 %  A description of each parameter follows:
 %
-%    o length: The string length.
+%    o length: the string length.
 %
 */
 MagickExport StringInfo *AcquireStringInfo(const size_t length)
@@ -248,7 +248,7 @@ MagickExport char *CloneString(char **destination,const char *source)
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
 */
 MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
@@ -262,7 +262,7 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
   clone_info=AcquireStringInfo(string_info->length);
   if (string_info->length != 0)
     (void) CopyMagickMemory(clone_info->datum,string_info->datum,
-      string_info->length);
+      string_info->length+1);
   return(clone_info);
 }
 
@@ -287,9 +287,9 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
 %
 %  A description of each parameter follows:
 %
-%    o target: The target string.
+%    o target: the target string.
 %
-%    o source: The source string.
+%    o source: the source string.
 %
 */
 
@@ -342,11 +342,11 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 %
 %  A description of each parameter follows:
 %
-%    o destination: The destination string.
+%    o destination: the destination string.
 %
-%    o source: The source string.
+%    o source: the source string.
 %
-%    o length: The length of the destination string.
+%    o length: the length of the destination string.
 %
 */
 MagickExport size_t ConcatenateMagickString(char *destination,
@@ -467,9 +467,9 @@ MagickExport MagickBooleanType ConcatenateString(char **destination,
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
-%    o source: The source string.
+%    o source: the source string.
 %
 */
 MagickExport void ConcatenateStringInfo(StringInfo *string_info,
@@ -511,7 +511,7 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The filename.
+%    o filename: the filename.
 %
 */
 MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
@@ -663,11 +663,11 @@ MagickExport char *ConstantString(const char *source)
 %
 %  A description of each parameter follows:
 %
-%    o destination: The destination string.
+%    o destination: the destination string.
 %
-%    o source: The source string.
+%    o source: the source string.
 %
-%    o length: The length of the destination string.
+%    o length: the length of the destination string.
 %
 */
 MagickExport size_t CopyMagickString(char *destination,const char *source,
@@ -713,7 +713,7 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
     }
   if (length != 0)
     *q='\0';
-  while (*p++ != '\0');
+  while (*p++ != '\0') ;
   return((size_t) (p-source-1));
 }
 
@@ -736,7 +736,7 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
 %
 %  A description of each parameter follows:
 %
-%    o string: The string.
+%    o string: the string.
 %
 */
 MagickExport char *DestroyString(char *string)
@@ -763,7 +763,7 @@ MagickExport char *DestroyString(char *string)
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
 */
 MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
@@ -798,7 +798,7 @@ MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
 %
 %  A description of each parameter follows:
 %
-%    o list: The string list.
+%    o list: the string list.
 %
 */
 MagickExport char **DestroyStringList(char **list)
@@ -839,7 +839,7 @@ MagickExport char **DestroyStringList(char **list)
 %
 %    o source: A character string.
 %
-%    o escape: The quoted string termination character to escape (e.g. '"').
+%    o escape: the quoted string termination character to escape (e.g. '"').
 %
 */
 MagickExport char *EscapeString(const char *source,const char escape)
@@ -907,7 +907,7 @@ MagickExport char *EscapeString(const char *source,const char escape)
 %
 %  A description of each parameter follows:
 %
-%    o filename: The filename.
+%    o filename: the filename.
 %
 %    o extent: Maximum length of the string.
 %
@@ -946,7 +946,7 @@ MagickExport char *FileToString(const char *filename,const size_t extent,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The filename.
+%    o filename: the filename.
 %
 %    o extent: Maximum length of the string.
 %
@@ -1004,16 +1004,21 @@ MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
   double
     length;
 
+  long
+    count;
+
   register long
     i;
 
   static const char
-    *units[7] = { "b", "kb", "mb", "gb", "tb", "pb", "eb" };
+    *units[] = { "b", "kb", "mb", "gb", "tb", "pb", "eb", (char *) NULL };
 
-  length=(double) ((MagickOffsetType) (size/2))*2.0;
-  for (i=0; (length > 1024.0) && (i < 6); i++)
+  length=(double) size;
+  for (i=0; (length >= 1024.0) && (units[i+1] != (const char *) NULL); i++)
     length/=1024.0;
-  return(FormatMagickString(format,MaxTextExtent,"%g%s",length,units[i]));
+  count=FormatMagickString(format,MaxTextExtent,"%.*g%s",(int) (i+2),length,
+    units[i]);
+  return(count);
 }
 
 /*
@@ -1039,7 +1044,7 @@ MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
 %   o string:  FormatMagickString() returns the formatted string in this
 %     character buffer.
 %
-%   o length: The maximum length of the string.
+%   o length: the maximum length of the string.
 %
 %   o format:  A string describing the format to use to write the remaining
 %     arguments.
@@ -1049,23 +1054,16 @@ MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
 MagickExport long FormatMagickStringList(char *string,const size_t length,
   const char *format,va_list operands)
 {
-  char
-    *locale;
-
   int
     n;
 
-  locale=ConstantString(setlocale(LC_ALL,(const char *) NULL));
-  (void) setlocale(LC_ALL,"C");
-#if defined(HAVE_VSNPRINTF)
+#if defined(MAGICKCORE_HAVE_VSNPRINTF)
   n=vsnprintf(string,length,format,operands);
 #else
   n=vsprintf(string,format,operands);
 #endif
   if (n < 0)
     string[length-1]='\0';
-  (void) setlocale(LC_ALL,locale);
-  locale=DestroyString(locale);
   return((long) n);
 }
 
@@ -1100,7 +1098,7 @@ MagickExport long FormatMagickString(char *string,const size_t length,
 %
 %  The format of the FormatMagickTime method is:
 %
-%      size_t FormatMagickTime(const time_t time,const size_t length,
+%      long FormatMagickTime(const time_t time,const size_t length,
 %        char *timestamp)
 %
 %  A description of each parameter follows.
@@ -1108,7 +1106,7 @@ MagickExport long FormatMagickString(char *string,const size_t length,
 %   o time:  the time since the Epoch (00:00:00 UTC, January 1, 1970),
 %     measured in seconds.
 %
-%   o length: The maximum length of the string.
+%   o length: the maximum length of the string.
 %
 %   o timestamp:  Return the Internet date/time here.
 %
@@ -1120,23 +1118,23 @@ MagickExport long FormatMagickTime(const time_t time,const size_t length,
     count;
 
   struct tm
-    local_meridian,
-    utc_meridian;
+    local_time,
+    utc_time;
 
   time_t
-    offset;
+    timezone;
 
   assert(timestamp != (char *) NULL);
-  local_meridian=(*localtime(&time));
-  utc_meridian=(*gmtime(&time));
-  offset=(time_t) ((local_meridian.tm_min-utc_meridian.tm_min)/60+
-    local_meridian.tm_hour-utc_meridian.tm_hour+24*((local_meridian.tm_year-
-    utc_meridian.tm_year) != 0 ? (local_meridian.tm_year-utc_meridian.tm_year) :
-    (local_meridian.tm_yday-utc_meridian.tm_yday)));
+  local_time=(*localtime(&time));
+  utc_time=(*gmtime(&time));
+  timezone=(time_t) ((local_time.tm_min-utc_time.tm_min)/60+local_time.tm_hour-
+    utc_time.tm_hour+24*((local_time.tm_year-utc_time.tm_year) != 0 ?
+    (local_time.tm_year-utc_time.tm_year) : (local_time.tm_yday-
+    utc_time.tm_yday)));
   count=FormatMagickString(timestamp,length,
-    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_meridian.tm_year+1900,
-    local_meridian.tm_mon+1,local_meridian.tm_mday,local_meridian.tm_hour,
-    local_meridian.tm_min,local_meridian.tm_sec,(long) offset);
+    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_time.tm_year+1900,
+    local_time.tm_mon+1,local_time.tm_mday,local_time.tm_hour,local_time.tm_min,
+    local_time.tm_sec,(long) timezone);
   return(count);
 }
 
@@ -1298,7 +1296,7 @@ MagickExport long LocaleCompare(const char *p,const char *q)
     return(-1);
   if (q == (char *) NULL)
     return(1);
-#if defined(HAVE_STRCASECMP)
+#if defined(MAGICKCORE_HAVE_STRCASECMP)
   return((long) strcasecmp(p,q));
 #else
   {
@@ -1385,7 +1383,7 @@ MagickExport void LocaleLower(char *string)
 %
 %    o q: A pointer to a character string to compare to p.
 %
-%    o length: The number of characters to compare in strings p & q.
+%    o length: the number of characters to compare in strings p & q.
 %
 */
 MagickExport long LocaleNCompare(const char *p,const char *q,
@@ -1395,7 +1393,7 @@ MagickExport long LocaleNCompare(const char *p,const char *q,
     return(-1);
   if (q == (char *) NULL)
     return(1);
-#if defined(HAVE_STRNCASECMP)
+#if defined(MAGICKCORE_HAVE_STRNCASECMP)
   return((long) strncasecmp(p,q,length));
 #else
   {
@@ -1475,11 +1473,11 @@ MagickExport void LocaleUpper(char *string)
 %
 %  A description of each parameter follows:
 %
-%    o file: The file, typically stdout.
+%    o file: the file, typically stdout.
 %
-%    o id: The string id.
+%    o id: the string id.
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
 */
 MagickExport void PrintStringInfo(FILE *file,const char *id,
@@ -1562,7 +1560,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
 */
 MagickExport void ResetStringInfo(StringInfo *string_info)
@@ -1592,9 +1590,9 @@ MagickExport void ResetStringInfo(StringInfo *string_info)
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
-%    o source: The source string.
+%    o source: the source string.
 %
 */
 MagickExport void SetStringInfo(StringInfo *string_info,
@@ -1605,6 +1603,8 @@ MagickExport void SetStringInfo(StringInfo *string_info,
   assert(string_info->signature == MagickSignature);
   assert(source != (StringInfo *) NULL);
   assert(source->signature == MagickSignature);
+  if (string_info->length == 0)
+    return;
   (void) ResetMagickMemory(string_info->datum,0,string_info->length);
   (void) CopyMagickMemory(string_info->datum,source->datum,MagickMin(
     string_info->length,source->length));
@@ -1631,9 +1631,9 @@ MagickExport void SetStringInfo(StringInfo *string_info,
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
-%    o source: The source string.
+%    o source: the source string.
 %
 */
 MagickExport void SetStringInfoDatum(StringInfo *string_info,
@@ -1665,9 +1665,9 @@ MagickExport void SetStringInfoDatum(StringInfo *string_info,
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
-%    o length: The string length.
+%    o length: the string length.
 %
 */
 MagickExport void SetStringInfoLength(StringInfo *string_info,
@@ -1708,9 +1708,9 @@ MagickExport void SetStringInfoLength(StringInfo *string_info,
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
-%    o path: The path.
+%    o path: the path.
 %
 */
 MagickExport void SetStringInfoPath(StringInfo *string_info,const char *path)
@@ -1741,7 +1741,7 @@ MagickExport void SetStringInfoPath(StringInfo *string_info,const char *path)
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string info.
+%    o string_info: the string info.
 %
 */
 MagickExport StringInfo *SplitStringInfo(StringInfo *string_info,
@@ -1768,40 +1768,6 @@ MagickExport StringInfo *SplitStringInfo(StringInfo *string_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S t r i n g T o S t r i n g I n f o                                       %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  StringToStringInfo() returns the contents of a file as a string.
-%
-%  The format of the StringToStringInfo method is:
-%
-%      StringInfo *StringToStringInfo(const char *string)
-%
-%  A description of each parameter follows:
-%
-%    o string:  The string.
-%
-*/
-MagickExport StringInfo *StringToStringInfo(const char *string)
-{
-  StringInfo
-    *string_info;
-
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(string != (const char *) NULL);
-  string_info=AcquireStringInfo(strlen(string)+1);
-  SetStringInfoDatum(string_info,(const unsigned char *) string);
-  return(string_info);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   S t r i n g I n f o T o S t r i n g                                       %
 %                                                                             %
 %                                                                             %
@@ -1816,7 +1782,7 @@ MagickExport StringInfo *StringToStringInfo(const char *string)
 %
 %  A description of each parameter follows:
 %
-%    o string_info: The string.
+%    o string_info: the string.
 %
 */
 MagickExport char *StringInfoToString(const StringInfo *string_info)
@@ -1831,10 +1797,9 @@ MagickExport char *StringInfoToString(const StringInfo *string_info)
   length=string_info->length;
   if (~length >= MaxTextExtent)
     string=(char *) AcquireQuantumMemory(length+MaxTextExtent,sizeof(*string));
-  if (string == (char *) NULL)
-    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) CopyMagickString(string,(char *) string_info->datum,
-    (length+1)*sizeof(*string));
+  if (string != (char *) NULL)
+    (void) CopyMagickString(string,(char *) string_info->datum,
+      (length+1)*sizeof(*string));
   return(string);
 }
 
@@ -1890,14 +1855,14 @@ MagickExport char **StringToArgv(const char *text,int *argc)
       p++;
     (*argc)++;
     if (*p == '"')
-      for (p++; (*p != '"') && (*p != '\0'); p++);
+      for (p++; (*p != '"') && (*p != '\0'); p++) ;
     if (*p == '\'')
-      for (p++; (*p != '\'') && (*p != '\0'); p++);
+      for (p++; (*p != '\'') && (*p != '\0'); p++) ;
     while ((isspace((int) ((unsigned char) *p)) == 0) && (*p != '\0'))
       p++;
   }
   (*argc)++;
-  argv=(char **) AcquireQuantumMemory((size_t) *argc+1UL,sizeof(*argv));
+  argv=(char **) AcquireQuantumMemory((size_t) (*argc+1UL),sizeof(*argv));
   if (argv == (char **) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConvertStringToARGV");
   /*
@@ -1913,12 +1878,12 @@ MagickExport char **StringToArgv(const char *text,int *argc)
     if (*q == '"')
       {
         p++;
-        for (q++; (*q != '"') && (*q != '\0'); q++);
+        for (q++; (*q != '"') && (*q != '\0'); q++) ;
       }
     else
       if (*q == '\'')
         {
-          for (q++; (*q != '\'') && (*q != '\0'); q++);
+          for (q++; (*q != '\'') && (*q != '\0'); q++) ;
           q++;
         }
       else
@@ -1978,8 +1943,19 @@ MagickExport double StringToDouble(const char *string,const double interval)
 
   assert(string != (char *) NULL);
   value=strtod(string,&q);
-  if (strchr(q,'%') != (char *) NULL)
-    value*=interval/100.0;
+  switch (tolower((int) ((unsigned char) *q)))
+  {
+    case '%': value*=pow(1024.0,0)*interval/100.0; break;
+    case 'k': value*=pow(1024.0,1); break;
+    case 'm': value*=pow(1024.0,2); break;
+    case 'g': value*=pow(1024.0,3); break;
+    case 't': value*=pow(1024.0,4); break;
+    case 'p': value*=pow(1024.0,5); break;
+    case 'e': value*=pow(1024.0,6); break;
+    case 'z': value*=pow(1024.0,7); break;
+    case 'y': value*=pow(1024.0,8); break;
+    default:  break;
+  }
   return(value);
 }
 
@@ -1988,6 +1964,82 @@ MagickExport double StringToDouble(const char *string,const double interval)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S t r i n g I n f o T o H e x S t r i n g                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  StringInfoToHexString() converts a string info string to a C string.
+%
+%  The format of the StringInfoToHexString method is:
+%
+%      char *StringInfoToHexString(const StringInfo *string_info)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string.
+%
+*/
+MagickExport char *StringInfoToHexString(const StringInfo *string_info)
+{
+  char
+    *string;
+
+  register const unsigned char
+    *p;
+
+  register long
+    i;
+
+  register unsigned char
+    *q;
+
+  size_t
+    length;
+
+  unsigned char
+    hex_digits[16];
+
+  length=string_info->length;
+  if (~length < MaxTextExtent)
+    ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
+  string=(char *) AcquireQuantumMemory(length+MaxTextExtent,2*sizeof(*string));
+  if (string == (char *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
+  hex_digits[0]='0';
+  hex_digits[1]='1';
+  hex_digits[2]='2';
+  hex_digits[3]='3';
+  hex_digits[4]='4';
+  hex_digits[5]='5';
+  hex_digits[6]='6';
+  hex_digits[7]='7';
+  hex_digits[8]='8';
+  hex_digits[9]='9';
+  hex_digits[10]='a';
+  hex_digits[11]='b';
+  hex_digits[12]='c';
+  hex_digits[13]='d';
+  hex_digits[14]='e';
+  hex_digits[15]='f';
+  p=string_info->datum;
+  q=(unsigned char *) string;
+  for (i=0; i < (long) string_info->length; i++)
+  {
+    *q++=hex_digits[(*p >> 4) & 0x0f];
+    *q++=hex_digits[*p & 0x0f];
+    p++;
+  }
+  *q='\0';
+  return(string);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S t r i n g T o k e n                                                     %
 %                                                                             %
 %                                                                             %
@@ -2186,6 +2238,40 @@ MagickExport char **StringToList(const char *text)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S t r i n g T o S t r i n g I n f o                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  StringToStringInfo() returns the contents of a file as a string.
+%
+%  The format of the StringToStringInfo method is:
+%
+%      StringInfo *StringToStringInfo(const char *string)
+%
+%  A description of each parameter follows:
+%
+%    o string:  The string.
+%
+*/
+MagickExport StringInfo *StringToStringInfo(const char *string)
+{
+  StringInfo
+    *string_info;
+
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(string != (const char *) NULL);
+  string_info=AcquireStringInfo(strlen(string)+1);
+  SetStringInfoDatum(string_info,(const unsigned char *) string);
+  return(string_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S t r i p S t r i n g                                                     %
 %                                                                             %
 %                                                                             %
@@ -2256,7 +2342,7 @@ MagickExport void StripString(char *message)
 %
 %  A description of each parameter follows:
 %
-%    o buffer: The buffer to perform replacements on. Replaced with new
+%    o buffer: the buffer to perform replacements on. Replaced with new
 %      allocation if a replacement is made.
 %
 %    o search: String to search for.
@@ -2297,7 +2383,7 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
     return(MagickFalse);
   result=(char *) NULL;
   length=strlen(source);
-  if (~length >= MaxTextExtent) 
+  if (~length >= MaxTextExtent)
     result=(char *) AcquireQuantumMemory(length+MaxTextExtent,sizeof(*result));
   if (result == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
@@ -2336,10 +2422,9 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
         result=(char *) ResizeQuantumMemory(result,length+MaxTextExtent,
           sizeof(*result));
         if (result == (char *) NULL)
-          ThrowFatalException(ResourceLimitFatalError,
-            "UnableToAcquireString");
+          ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
       }
-    (void) ConcatenateMagickString(result+destination_offset,replace,
+    (void) ConcatenateMagickString(result+destination_offset,replace,(size_t)
       (length+MaxTextExtent)*sizeof(*result));
     destination_offset+=replace_length;
     /*
@@ -2360,8 +2445,7 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
       result=(char *) ResizeQuantumMemory(result,length+MaxTextExtent,
         sizeof(*result));
       if (result == (char *) NULL)
-        ThrowFatalException(ResourceLimitFatalError,
-          "UnableToAcquireString");
+        ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
     }
   (void) ConcatenateMagickString(result+destination_offset,source,(size_t)
     (length+MaxTextExtent-destination_offset)*sizeof(*result));
