@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -26,35 +26,43 @@ extern "C" {
   Define declarations.
 */
 #define MagickPackageName "ImageMagick"
-#define MagickCopyright  "Copyright (C) 1999-2007 ImageMagick Studio LLC"
-#define MagickLibVersion  0x636
-#define MagickLibVersionText  "6.3.6"
-#define MagickLibVersionNumber  6,3,6,10
-#if (QuantumDepth == 8)
+#define MagickCopyright  "Copyright (C) 1999-2008 ImageMagick Studio LLC"
+#define MagickLibVersion  0x645
+#define MagickLibVersionText  "6.4.5"
+#define MagickLibVersionNumber  1,0,0
+#define MagickReleaseDate  "2008-11-01"
+#define MagickChangeDate   "20081101"
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
 #define MagickQuantumDepth  "Q8"
 #define MagickQuantumRange  "255"
-#elif (QuantumDepth == 16)
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
 #define MagickQuantumDepth  "Q16"
 #define MagickQuantumRange  "65535"
-#elif (QuantumDepth == 32)
+#elif (MAGICKCORE_QUANTUM_DEPTH == 32)
 #define MagickQuantumDepth  "Q32"
 #define MagickQuantumRange  "4294967295"
-#elif (QuantumDepth == 64)
+#elif (MAGICKCORE_QUANTUM_DEPTH == 64)
 #define MagickQuantumDepth  "Q64"
 #define MagickQuantumRange  "18446744073709551615"
 #else
 #define MagickQuantumDepth  "Q?"
 #define MagickQuantumRange  "?"
 #endif
-#define MagickReleaseDate  "11/06/07"
-#if !defined(UseHDRI)
-#define MagickVersion MagickPackageName " " MagickLibVersionText " " \
-  MagickReleaseDate " " MagickQuantumDepth " http://www.imagemagick.org"
+#if !defined(MAGICKCORE_HDRI_SUPPORT)
+#define MagickHDRISupport ""
 #else
-#define MagickVersion MagickPackageName " " MagickLibVersionText " " \
-  MagickReleaseDate " " MagickQuantumDepth " HDRI http://www.imagemagick.org"
+#define MagickHDRISupport "HDRI "
 #endif
-#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.3.6/index.html"
+#if !defined(_OPENMP)
+#define MagickOPENMPSupport ""
+#else
+#define MagickOPENMPSupport "OpenMP "
+#endif
+#define MagickSupport MagickHDRISupport MagickOPENMPSupport
+#define MagickVersion MagickPackageName " " MagickLibVersionText " " \
+  MagickReleaseDate " " MagickQuantumDepth " " MagickSupport \
+  "http://www.imagemagick.org"
+#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.4.5/index.html"
 
 extern MagickExport char
   *GetMagickHomeURL(void);
