@@ -17,7 +17,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -86,9 +86,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o argc: The number of elements in the argument vector.
+%    o argc: the number of elements in the argument vector.
 %
 %    o argv: A text array containing the command line arguments.
 %
@@ -96,7 +96,6 @@
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 static void ConjureUsage(void)
@@ -105,16 +104,22 @@ static void ConjureUsage(void)
     **p;
 
   static const char
-    *settings[]=
+    *miscellaneous[]=
     {
       "-debug events        display copious debugging information",
       "-help                print program options",
+      "-list type           print a list of supported option arguments",
       "-log format          format of debugging information",
+      "-version             print version information",
+      (char *) NULL
+    },
+    *settings[]=
+    {
       "-monitor             monitor progress",
       "-quiet               suppress all warning messages",
       "-regard-warnings     pay attention to warning messages",
+      "-seed value          seed a new sequence of pseudo-random numbers",
       "-verbose             print detailed information about the image",
-      "-version             print version information",
       (char *) NULL
     };
 
@@ -125,6 +130,9 @@ static void ConjureUsage(void)
   (void) printf("\nImage Settings:\n");
   for (p=settings; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
+  (void) printf("\nMiscellaneous Options:\n");
+  for (p=miscellaneous; *p != (char *) NULL; p++)
+    (void) printf("  %s\n",*p);
   (void) printf("\nIn additiion, define any key value pairs required by "
     "your script.  For\nexample,\n\n");
   (void) printf("    conjure -size 100x100 -color blue -foo bar script.msl\n");
@@ -143,9 +151,8 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
 }
 #define ThrowConjureException(asperity,tag,option) \
 { \
-  if (exception->severity < (asperity)) \
-    (void) ThrowMagickException(exception,GetMagickModule(),asperity,tag, \
-      "`%s'",option); \
+  (void) ThrowMagickException(exception,GetMagickModule(),asperity,tag,"`%s'", \
+     option); \
   DestroyConjure(); \
   return(MagickFalse); \
 }
@@ -191,15 +198,8 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowConjureException(ResourceLimitError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowConjureException(ResourceLimitError,"MemoryAllocationFailed",
+      strerror(errno));
   for (i=1; i < (long) argc; i++)
   {
     option=argv[i];
@@ -246,6 +246,17 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
           continue;
         if (LocaleCompare("regard-warnings",option+1) == 0)
           break;
+        if (LocaleCompare("seed",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConjureException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConjureInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("verbose",option+1) == 0)
           {
             image_info->verbose=(*option == '-') ? MagickTrue : MagickFalse;
@@ -279,7 +290,7 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
       ThrowConjureException(ImageError,"UnableToPersistKey",argv[i]);
     (void) FormatMagickString(image_info->filename,MaxTextExtent,"msl:%s",
       argv[i]);
-    image=ReadImage(image_info,exception);
+    image=ReadImages(image_info,exception);
     CatchException(exception);
     status&=image != (Image *) NULL;
     if (image != (Image *) NULL)
